import 'package:flutter/material.dart';

import 'package:loyalty_management_tool/models/postlist.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:loyalty_management_tool/utils/app_buttons.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:timeago/timeago.dart' as timeago;

import 'appHtmlViewer.dart';

// ignore: must_be_immutable
class PostCardFooter extends StatefulWidget {
  Content? contentData;
  Statistics? stats;
  bool isRewards;
  RewardsItem data;
  Function? commentCallback;
  bool? isDetailPage;
  int? dateTime;
  String? postType;
  Function? onAnswerClickCallback;
  Function? onVoteCallback;
  int? postId;
  bool isVoted;
  int? postOwnerTypeId;
  SharedPreferences? prefs;
  bool? isNewsPage;
  String? webLink;
  bool? isFilterPage;
  Function? onSubmitAnswer;
  String? searchHighlightWord;
  GlobalKey? globalKeyHtmlWidget;
  String? welcomeMessageData;
  bool isWelcomeChat;

  PostCardFooter(
      {this.contentData,
      this.stats,
      required this.data,
      this.isRewards = false,
      this.isWelcomeChat = false,
      this.welcomeMessageData,
      this.globalKeyHtmlWidget,
      this.commentCallback,
      this.isDetailPage,
      this.dateTime,
      this.postType,
      this.postId,
      this.isFilterPage = false,
      this.onVoteCallback,
      this.isVoted = false,
      this.postOwnerTypeId,
      this.prefs,
      this.isNewsPage,
      this.webLink,
      this.onSubmitAnswer,
      this.searchHighlightWord,
      this.onAnswerClickCallback});

  @override
  _PostCardFooter createState() => _PostCardFooter(
      contentData: contentData,
      stats: stats,
      isDetailPage: isDetailPage,
      commentCallback: commentCallback,
      dateTime: dateTime,
      postId: postId,
      postType: postType,
      prefs: prefs,
      postOwnerTypeId: postOwnerTypeId,
      onVoteCallback: onVoteCallback,
      isVoted: isVoted,
      webLink: webLink,
      onSubmitAnswer: onSubmitAnswer,
      onAnswerClickCallback: onAnswerClickCallback);
}

class _PostCardFooter extends State<PostCardFooter> {
  late TextStyleElements styleElements;
  Statistics? stats;
  Content? contentData;
  int? comments;
  Function? commentCallback;
  bool? isDetailPage;
  int? dateTime;
  String? postType;
  int? postId;
  Function? onVoteCallback;
  bool isVoted;
  Function? onAnswerClickCallback;
  int? postOwnerTypeId;
  SharedPreferences? prefs;
  String? webLink;
  Function? onSubmitAnswer;

  _PostCardFooter(
      {this.contentData,
      this.stats,
      this.commentCallback,
      this.isDetailPage = false,
      this.dateTime,
      this.postType,
      this.postId,
      this.onVoteCallback,
      this.isVoted = false,
      this.postOwnerTypeId,
      this.prefs,
      this.webLink,
      this.onSubmitAnswer,
      this.onAnswerClickCallback});

  @override
  Widget build(BuildContext context) {
    print("refreshing post card footer");
    // Utility().screenUtilInit(context);
    // DateTime time =
    // print(timeago.format(time));
    styleElements = TextStyleElements(context);
    comments = stats != null
        ? stats!.commentCount != null
            ? stats!.commentCount
            : 0
        : 0;

    return Container(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Expanded(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Padding(
                  padding: EdgeInsets.only(top: 0.0, bottom: 8.0),
                  child: Row(
                    children: [
                      Visibility(
                        visible: dateTime != null,
                        child: Padding(
                          padding:
                              EdgeInsets.only(left: 8.0, bottom: 0, top: 4),
                          child: Text(
                            dateTime != null
                                ? timeago.format(
                                    DateTime.fromMillisecondsSinceEpoch(
                                        dateTime!))
                                : "",
                            style: styleElements
                                .captionThemeScalable(context)
                                .copyWith(
                                    fontSize: 10,
                                    color: HexColor(AppColors.appColorBlack35)),
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
                Visibility(
                  visible: (widget.data.title != null),
                  child: Text(
                    widget.data.title ?? "",
                    maxLines: 2,
                    overflow: TextOverflow.ellipsis,
                    style: styleElements
                        .subtitle1ThemeScalable(context)
                        .copyWith(fontWeight: FontWeight.bold),
                  ),
                ),
                Visibility(
                    visible: (widget.data.subtitle != null),
                    child: Container(
                        child: Padding(
                      padding: EdgeInsets.only(top: 8.0),
                      // child: Text(
                      child: AppHtmlViewer(
                        key: widget.globalKeyHtmlWidget,
                        isDetailPage: widget.isDetailPage,
                        sourceString: widget.data.subtitle,
                      ),
                    )))
              ],
            ),
          ),
          Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              SizedBox(
                width: 100,
                child: appElevatedButton(
                  onPressed: () async {},
                  color: HexColor(AppColors.appColorTeel),
                  child: Wrap(
                    direction: Axis.horizontal,
                    crossAxisAlignment: WrapCrossAlignment.center,
                    children: [
                      Icon(Icons.gamepad_outlined,
                          color: HexColor(AppColors.appColorYellow)),
                      Text(
                        widget.data.incentiveCoins.toString(),
                        style: styleElements
                            .subtitle2ThemeScalable(context)
                            .copyWith(color: HexColor(AppColors.appColorWhite)),
                      ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                width: 100,
                child: appElevatedButton(
                  onPressed: () async {},
                  color: HexColor(AppColors.appColorTeel),
                  child: Wrap(
                    direction: Axis.horizontal,
                    crossAxisAlignment: WrapCrossAlignment.center,
                    children: [
                      Icon(Icons.celebration,
                          color: HexColor(AppColors.appColorYellow)),
                      Text(
                        num.parse(widget.data.incentiveCash ?? "0")
                            .toInt()
                            .toString(),
                        style: styleElements
                            .subtitle2ThemeScalable(context)
                            .copyWith(color: HexColor(AppColors.appColorWhite)),
                      ),
                    ],
                  ),
                ),
              )
            ],
          ),
        ],
      ),
    );
  }

  getlistofkeywords() {
    if (contentData != null &&
        contentData!.keywords != null &&
        contentData!.keywords!.length > 0) {
      List<Widget> list = [];
      for (int i = 0; i < contentData!.keywords!.length; i++) {
        list.add(Padding(
          padding: const EdgeInsets.all(4.0),
          child: Chip(
            label: Text(
              contentData!.keywords![i].toString(),
              style: styleElements.captionThemeScalable(context),
            ),
            backgroundColor: HexColor(AppColors.appColorGrey300),
          ),
        ));
      }
      return list;
    } else {
      List<Widget> list = [];
      list.add(Text(''));
      return list;
    }
  }

  // getlistofmentions() {
  //   if (contentData.mentions != null && contentData.mentions.length > 0) {
  //     List<Widget> list = [];
  //     for (int i = 0; i < contentData.mentions.length; i++) {
  //       list.add(Padding(
  //         padding: const EdgeInsets.all(4.0),
  //         child: Chip(
  //           label: Text(
  //             '@'+contentData.mentions[i].toString(),
  //             style: styleElements.captionThemeScalable(context),
  //           ),
  //           backgroundColor: HexColor(AppColors.appMainColor33)[300],
  //         ),
  //       ));
  //     }
  //     return list;
  //   } else {
  //     List<Widget> list = [];
  //     list.add(Text(''));
  //     return list;
  //   }
  // }

  String getHost(String? path) {
    try {
      if (path != null)
        return Uri.parse(path).host;
      else
        return "";
    } catch (e) {
      print(e);
      return "";
    }
  }
}
