import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:loyalty_management_tool/utils/text_styling.dart';

import '../../models/bank_details_response.dart';
import '../../models/base_res.dart';
import '../../models/location_reponse_data.dart';
import '../../models/subscription_view_response.dart';
import '../../notifications/address_notification.dart';
import '../../utils/basicInfo.dart';
import '../../utils/empty_widget.dart';
import '../../utils/localization.dart';
import '../cubits/cubit_main.dart';
import '../states/main_state.dart';
import '../states/states.dart';
import 'loading_indicator.dart';

class AccountDetailsBillingWidget extends StatefulWidget {
  final String? avatar;
  final String? title;
  final String? subTitle;

  AccountDetailsBillingWidget(
      {Key? key, this.avatar, this.title, this.subTitle})
      : super(key: key);

  @override
  AccountDetailsBillingWidgetState createState() =>
      AccountDetailsBillingWidgetState();
}

class AccountDetailsBillingWidgetState
    extends State<AccountDetailsBillingWidget> {
  BasicInfo? basicInfo = BasicInfo();

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      dataFetch();
    });
  }

  Future<void> dataFetch() async {
    BlocProvider.of<CubitMain>(context)
        .getData(getBody(), context, basicInfo!.subscriptionView, getData);
  }

  String getBody() {
    return jsonEncode({
      "person_id": basicInfo!.userId,
      "app_type": "FOOPWORKS",
      "business_id": basicInfo!.businessId
    });
  }

  dynamic getData(dynamic data) {
    return SubscriptionViewResponse.fromJson(data);
  }

  TextStyleElements? styleElements;

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);
      if (state is LoadingState) {
        return LoadingIndicator();
      } else if (state is LoadedWithData) {
        return Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Expanded(
              child: Padding(
                padding: const EdgeInsets.only(
                    top: 8.0, bottom: 8.0, left: 8, right: 8),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          margin: const EdgeInsets.only(top: 20),
                          child: Text(
                            "Account Number",
                            maxLines: 1,
                            style:
                                styleElements!.bodyText2ThemeScalable(context),
                            textAlign: TextAlign.left,
                          ),
                        )),
                    Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          margin: const EdgeInsets.only(
                            top: 8,
                          ),
                          child: Text(
                            state.data.accountingDetails != null
                                ? state.data.accountingDetails.accountNumber ??
                                    "--"
                                : "--",
                            style: styleElements!
                                .bodyText1ThemeScalable(context)
                                .copyWith(fontWeight: FontWeight.w600),
                            textAlign: TextAlign.left,
                          ),
                        )),
                    Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          margin: const EdgeInsets.only(top: 20),
                          child: Text(
                            "GSTN",
                            maxLines: 1,
                            style:
                                styleElements!.bodyText2ThemeScalable(context),
                            textAlign: TextAlign.left,
                          ),
                        )),
                    Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          margin: const EdgeInsets.only(right: 16),
                          child: Text(
                            state.data.accountingDetails != null
                                ? state.data.accountingDetails.gst ?? "--"
                                : "--",
                            style: styleElements!
                                .bodyText1ThemeScalable(context)
                                .copyWith(fontWeight: FontWeight.w600),
                            textAlign: TextAlign.left,
                          ),
                        )),
                  ],
                ),
              ),
            ),
            Expanded(
              child: Align(
                alignment: Alignment.centerRight,
                child: Padding(
                  padding: const EdgeInsets.only(
                      top: 8.0, bottom: 8.0, right: 8.0, left: 20),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Align(
                          alignment: Alignment.topLeft,
                          child: Container(
                            margin: const EdgeInsets.only(top: 20),
                            child: Text(
                              "Billing address",
                              maxLines: 1,
                              style: styleElements!
                                  .bodyText2ThemeScalable(context),
                              textAlign: TextAlign.left,
                            ),
                          )),
                      Align(
                          alignment: Alignment.topLeft,
                          child: Container(
                            margin: const EdgeInsets.only(
                              top: 8,
                            ),
                            child: Text(
                              state.data.accountingDetails != null
                                  ? state.data.accountingDetails
                                          .billingAddress ??
                                      "--"
                                  : "--",
                              style: styleElements!
                                  .bodyText2ThemeScalable(context)
                                  .copyWith(fontWeight: FontWeight.w600),
                              textAlign: TextAlign.left,
                            ),
                          )),
                    ],
                  ),
                ),
              ),
            ),
          ],
        );
      } else {
        return Container(
          child: Center(
            child: EmptyWidget(
                message:
                    AppLocalizationsRewards.of(context)!.translate("no_data")),
          ),
        );
      }
    });
  }
}
