import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:intl/intl.dart';
import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';
import 'package:loyalty_management_tool/utils/toast_builder.dart';
import 'package:loyalty_management_tool/utils/utility_class.dart';
import 'package:shared_preferences/shared_preferences.dart';

class DateTimeWidget extends StatefulWidget {
  final String title;
  final String subTitle;
  final RewardsItem? item;
  final Function(num startTime, num? startDate) nextCallBack;

  const DateTimeWidget({
    Key? key,
    required this.subTitle,
    required this.title,
    this.item,
    required this.nextCallBack,
  }) : super(key: key);

  @override
  DateTimeWidgetState createState() => DateTimeWidgetState();
}

class DateTimeWidgetState extends State<DateTimeWidget> {
  GlobalKey<FormState> formKey = GlobalKey();

  late TextStyleElements styleElements;
  int selectedStartEpoch = 0;
  String selectedStartDate = "-";
  int selectedEndEpoch = 0;
  String selectedEndDate = "-";
  int selectedDueEpoch = 0;
  String selectedDueDate = "Due date";
  TimeOfDay? selectedStartTimeOfDay;
  String selectedStartTime = '-';
  TimeOfDay? selectedEndTimeOfDay;
  String selectedEndTime = '-';
  String? selectedPrivacyType;

  SharedPreferences? prefs = locatorRewards<SharedPreferences>();

  BasicInfo? basicInfo = BasicInfo();

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      if (widget.item != null) {
        final DateFormat formatter = DateFormat('dd-MM-yyyy');
        final DateFormat formatterTime = DateFormat('HH:mm:ss');
        final DateFormat formatterTimeBoth = DateFormat('dd-MM-yyyy HH:mm:ss');
        if (widget.item!.startDatetime != null) {
          var dateTime =
              DateTime.fromMillisecondsSinceEpoch(widget.item!.startDatetime!);
          final String formatted = formatter.format(dateTime);
          final String formattedT = formatterTime.format(dateTime);

          selectedStartDate = formatted;
          selectedStartTime = formattedT;
          selectedStartEpoch = widget.item!.startDatetime!;
          selectedStartTimeOfDay = TimeOfDay.fromDateTime(dateTime);
        }
        if (widget.item!.endDatetime != null) {
          var dateTime =
              DateTime.fromMillisecondsSinceEpoch(widget.item!.endDatetime!);
          final String formatted = formatter.format(dateTime);
          final String formattedT = formatterTime.format(dateTime);
          selectedEndDate = formatted;
          selectedEndTime = formattedT;
          selectedEndEpoch = widget.item!.endDatetime!;
          selectedEndTimeOfDay = TimeOfDay.fromDateTime(dateTime);
        }

        setState(() {});
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    final startDateWidget = GestureDetector(
        onTap: () {
          _selectStartDate(context);
        },
        child: Container(
            padding: const EdgeInsets.only(top: 16),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  //
                  color: HexColor(AppColors.appColorGrey50),
                  width: 1.0,
                ),
              ), // set border width
            ),
            child: Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  selectedStartDate != "-" ? selectedStartDate : "-",
                  textAlign: TextAlign.left,
                  style: styleElements.bodyText2ThemeScalable(context),
                ))));
    final endDateWidget = GestureDetector(
        onTap: () {
          _selectEndDate(context);
        },
        child: Container(
            padding: EdgeInsets.only(top: 16),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  //
                  color: HexColor(AppColors.appColorGrey50),
                  width: 1.0,
                ),
              ), // set border width
            ),
            child: Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  selectedEndDate != "-" ? selectedEndDate : "-",
                  textAlign: TextAlign.left,
                  style: styleElements.bodyText2ThemeScalable(context),
                ))));
    final startTimeWidget = GestureDetector(
        onTap: () {
          _selectStartTime(context);
        },
        child: Container(
            padding: EdgeInsets.only(top: 16),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  //
                  color: HexColor(AppColors.appColorGrey50),
                  width: 1.0,
                ),
              ), // set border width
            ),
            child: Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  selectedStartTime,
                  textAlign: TextAlign.left,
                  style: styleElements.bodyText2ThemeScalable(context),
                ))));
    final endTimeWidget = GestureDetector(
        onTap: () {
          _selectEndTime(context);
        },
        child: Container(
            padding: EdgeInsets.only(top: 16),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  //
                  color: HexColor(AppColors.appColorGrey50),
                  width: 1.0,
                ),
              ), // set border width
            ),
            child: Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  selectedEndTime,
                  textAlign: TextAlign.left,
                  style: styleElements.bodyText2ThemeScalable(context),
                ))));
    return Form(
      key: formKey,
      child: ListView(
        shrinkWrap: true,
        children: [
          Padding(
            padding: const EdgeInsets.only(
                bottom: 30.0, left: 16, right: 16, top: 8),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              //Center Row contents horizontally,
              crossAxisAlignment: CrossAxisAlignment.start,
              //Center Row contents vertically,

              children: [
                Expanded(
                  child: Align(
                    alignment: Alignment.center,
                    child: Container(
                      child: Text(
                        widget.title,
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(fontWeight: FontWeight.bold),
                      ),
                    ),
                  ),
                ),
                Align(
                  alignment: Alignment.centerRight,
                  child: InkWell(
                    onTap: () {
                      var ed;
                      if (selectedStartDate != "-" &&
                          selectedStartTime != "-") {
                        var d = DateTime.fromMillisecondsSinceEpoch(
                            selectedStartEpoch);
                        var sd = DateTime(
                            d.year,
                            d.month,
                            d.day,
                            selectedStartTimeOfDay!.hour,
                            selectedStartTimeOfDay!.minute);

                        if (selectedEndDate != "-" && selectedEndTime != "-") {
                          var e_d = DateTime.fromMillisecondsSinceEpoch(
                              selectedEndEpoch);
                          ed = DateTime(
                              e_d.year,
                              e_d.month,
                              e_d.day,
                              selectedEndTimeOfDay!.hour,
                              selectedEndTimeOfDay!.minute);
                        }

                        Navigator.pop(context);
                        widget.nextCallBack(sd.millisecondsSinceEpoch,
                            ed?.millisecondsSinceEpoch);
                      } else {
                        ToastBuilder().showToast(
                            AppLocalizationsRewards.of(context)!
                                .translate("select_start_date_time"),
                            context,
                            HexColor(AppColors.information));
                      }
                    },
                    child: Container(
                      child: Text(
                        AppLocalizationsRewards.of(context)!.translate("next"),
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(
                                fontWeight: FontWeight.bold,
                                color: HexColor(AppColors.blueApp)),
                      ),
                    ),
                  ),
                )
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(12),
            child: Text(
              widget.subTitle,
              textAlign: TextAlign.center,
              style: styleElements.captionThemeScalable(context),
            ),
          ),
          Padding(
              padding: EdgeInsets.only(bottom: 16, left: 8.0, right: 8),
              child: Row(
                children: [
                  Expanded(
                      child: getEventDetailCard(
                    'Start date',
                    startDateWidget,
                  )),
                  const SizedBox(
                    width: 100,
                  ),
                  Expanded(
                      child: getEventDetailCard(
                    'Start time',
                    startTimeWidget,
                  )),
                ],
              )),
          Visibility(
            visible: true,
            child: Padding(
                padding: const EdgeInsets.only(bottom: 16, left: 8.0, right: 8),
                child: Row(
                  children: [
                    Expanded(
                        child: getEventDetailCard(
                      'End date',
                      endDateWidget,
                    )),
                    const SizedBox(
                      width: 100,
                    ),
                    Expanded(
                        child: getEventDetailCard(
                      'End time',
                      endTimeWidget,
                    )),
                  ],
                )),
          ),
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Text(
              AppLocalizationsRewards.of(context)!
                  .translate("rewards_end_date_des"),
              style: styleElements.bodyText2ThemeScalable(context),
            ),
          ),
        ],
      ),
    );
  }

  Future<void> _selectStartDate(BuildContext context) async {
    var newDate;

    newDate = DateTime.now();

    final DateTime? picked = await showDatePicker(
        context: context,
        initialDate: newDate,
        firstDate: newDate,
        builder: (BuildContext context, Widget? child) {
          return Theme(
            data: ThemeData.light().copyWith(
              primaryColor: HexColor(AppColors.appColorBlack85),
              buttonTheme:
                  const ButtonThemeData(textTheme: ButtonTextTheme.primary),
              colorScheme: ColorScheme.dark(
                primary: HexColor(AppColors.appColorBlack85),
                onPrimary: HexColor(AppColors.appColorWhite),
                surface: HexColor(AppColors.appColorWhite),
                onSurface: HexColor(AppColors.appColorBlack85),
              ).copyWith(secondary: HexColor(AppColors.appColorBlack85)),
            ),
            child: child!,
          );
        },
        lastDate: DateTime(DateTime.now().year + 100));
    if (picked != null) {
      setState(() {
        selectedStartEpoch = picked.millisecondsSinceEpoch;
        selectedStartDate = DateFormat("dd-mm-yyyy").format(picked);
      });
    }
  }

  Future<void> _selectEndDate(BuildContext context) async {
    var newDate = new DateTime(
        DateTime.now().year, DateTime.now().month, DateTime.now().day);

    final DateTime? picked = await showDatePicker(
      context: context,
      initialDate: DateTime(newDate.year, newDate.month, newDate.day + 1),
      firstDate: DateTime(newDate.year, newDate.month, newDate.day),
      builder: (BuildContext context, Widget? child) {
        return Theme(
          data: ThemeData.light().copyWith(
            primaryColor: HexColor(AppColors.appColorBlack85),
            buttonTheme: ButtonThemeData(textTheme: ButtonTextTheme.primary),
            colorScheme: ColorScheme.dark(
              primary: HexColor(AppColors.appColorBlack85),
              onPrimary: HexColor(AppColors.appColorWhite),
              surface: HexColor(AppColors.appColorWhite),
              onSurface: HexColor(AppColors.appColorBlack85),
            ).copyWith(secondary: HexColor(AppColors.appColorBlack85)),
          ),
          child: child!,
        );
      },
      lastDate: DateTime(newDate.year + 100),
    );
    if (picked != null) {
      setState(() {
        selectedEndEpoch = picked.millisecondsSinceEpoch;
        selectedEndDate = DateFormat("dd-mm-yyyy").format(picked);
      });
    }
  }

  Future<void> _selectStartTime(BuildContext context) async {
    final TimeOfDay? picked = await showTimePicker(
      context: context,
      initialTime: TimeOfDay.now(),
      builder: (BuildContext context, Widget? child) {
        return Theme(
          data: ThemeData.light().copyWith(
            primaryColor: HexColor(AppColors.appColorBlack85),
            buttonTheme: ButtonThemeData(textTheme: ButtonTextTheme.primary),
            colorScheme: ColorScheme.dark(
              primary: HexColor(AppColors.appColorBlack85),
              onPrimary: HexColor(AppColors.appColorWhite),
              surface: HexColor(AppColors.appColorWhite),
              onSurface: HexColor(AppColors.appColorBlack85),
            ).copyWith(secondary: HexColor(AppColors.appColorBlack85)),
          ),
          child: child!,
        );
      },
    );
    if (picked != null) {
      setState(() {
        selectedStartTimeOfDay = picked;
        selectedStartTime = Utility().getDateFormat(
            "HH:mm:ss",
            DateTime(DateTime.now().year, DateTime.now().month,
                DateTime.now().day, picked.hour, picked.minute));
      });
    }
  }

  Widget getEventDetailCard(String title, Widget child, {EdgeInsets? padding}) {
    return Container(
        child: Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisSize: MainAxisSize.min,
      children: [
        Padding(
          padding: const EdgeInsets.all(2.0),
          child: Text(title),
        ),
        Padding(
          padding: padding ?? EdgeInsets.all(0),
          child: child,
        )
      ],
    ));
  }

  Future<void> _selectEndTime(BuildContext context) async {
    final TimeOfDay? picked = await showTimePicker(
      context: context,
      initialTime: TimeOfDay.now(),
      builder: (BuildContext context, Widget? child) {
        return Theme(
          data: ThemeData.light().copyWith(
            primaryColor: Colors.black,
            buttonTheme:
                const ButtonThemeData(textTheme: ButtonTextTheme.primary),
            colorScheme: const ColorScheme.dark(
              primary: Colors.black,
              onPrimary: Colors.white,
              surface: Colors.white,
              onSurface: Colors.black,
            ).copyWith(secondary: Colors.black),
          ),
          child: child!,
        );
      },
    );
    if (picked != null) {
      setState(() {
        selectedEndTimeOfDay = picked;
        selectedEndTime = Utility().getDateFormat(
            "HH:mm:ss",
            DateTime(DateTime.now().year, DateTime.now().month,
                DateTime.now().day, picked.hour, picked.minute));
      });
    }
  }
}
