import 'package:flutter/material.dart';
import 'package:lottie/lottie.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../../utils/app_buttons.dart';
import '../../utils/colors.dart';
import '../../utils/hexColors.dart';
import '../../utils/text_styling.dart';
import 'button_filled.dart';

// ignore: must_be_immutable
class CustomDialogue extends StatelessWidget {
  String title;
  String subtitle;
  late SharedPreferences prefs;
  String type;
  bool? isVerified;
  late TextStyleElements styleElements;

  Size displaySize(BuildContext context) {
    debugPrint('Size = ' + MediaQuery.of(context).size.toString());
    return MediaQuery.of(context).size;
  }

  double displayHeight(BuildContext context) {
    debugPrint('Height = ' + displaySize(context).height.toString());
    return displaySize(context).height;
  }

  double displayWidth(BuildContext context) {
    debugPrint('Width = ' + displaySize(context).width.toString());
    return displaySize(context).width;
  }

  void setSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
  }

  CustomDialogue(
      {Key? key,
      required this.type,
      required this.title,
      required this.subtitle,
      required this.isVerified})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    setSharedPreferences();
    return Dialog(
        shape:
            RoundedRectangleBorder(borderRadius: BorderRadius.circular(12.0)),
        child: Container(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(20.0),
                child: Align(
                  alignment: Alignment.topCenter,
                  child: SizedBox(
                    width: 200,
                    child: Lottie.asset(
                      'packages/loyalty_management_tool/assets/anim/sc.json',
                      width: 200,
                      height: 200,
                      fit: BoxFit.fill,
                    ),
                  ),
                ),
              ),
              Align(
                  alignment: Alignment.topCenter,
                  child: Container(
                    margin: const EdgeInsets.only(bottom: 12, top: 30),
                    child: Text(
                      title,
                      style: styleElements.headline6ThemeScalable(context),
                      textAlign: TextAlign.center,
                    ),
                  )),
              Align(
                  alignment: Alignment.topCenter,
                  child: Container(
                    margin: const EdgeInsets.only(bottom: 30),
                    child: Padding(
                      padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                      child: Text(
                        subtitle,
                        style: styleElements.captionThemeScalable(context),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  )),
              Padding(
                padding: const EdgeInsets.only(
                    top: 8.0, bottom: 8.0, left: 16, right: 16),
                child: Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Text(
                      "Transaction ID:",
                      maxLines: 2,
                      overflow: TextOverflow.ellipsis,
                      style: styleElements
                          .subtitle2ThemeScalable(context)
                          .copyWith(fontWeight: FontWeight.bold),
                    ),
                    Text("308009775679865",
                        maxLines: 2,
                        overflow: TextOverflow.ellipsis,
                        style: styleElements
                            .headline6ThemeScalableW(context)
                            .copyWith(fontWeight: FontWeight.bold)),
                  ],
                ),
              ),
              Padding(
                padding: const EdgeInsets.only(
                    top: 8.0, bottom: 8.0, left: 16, right: 16),
                child: Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Text(
                      "Amount paid:",
                      maxLines: 2,
                      overflow: TextOverflow.ellipsis,
                      style: styleElements
                          .subtitle2ThemeScalable(context)
                          .copyWith(fontWeight: FontWeight.bold),
                    ),
                    Text(
                        AppLocalizationsRewards.of(context)!
                                .translate("rupee_symbol") +
                            "7658",
                        maxLines: 2,
                        overflow: TextOverflow.ellipsis,
                        style: styleElements
                            .headline6ThemeScalableW(context)
                            .copyWith(fontWeight: FontWeight.bold)),
                  ],
                ),
              ),
              Visibility(
                visible: !isVerified!,
                child: Align(
                    alignment: Alignment.topCenter,
                    child: Container(
                      margin: const EdgeInsets.only(bottom: 20),
                      child: Text(
                        AppLocalizationsRewards.of(context)!
                            .translate("non_verified_person"),
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(fontWeight: FontWeight.bold),
                        textAlign: TextAlign.center,
                      ),
                    )),
              ),
              Padding(
                padding: const EdgeInsets.all(30.0),
                child: SizedBox(
                  height: 45,
                  child: Container(
                    child: appElevatedButton(
                      color: HexColor(AppColors.appMainColor),
                      onPressed: () async {},
                      shape: StadiumBorder(
                          side: BorderSide(
                              color: HexColor(AppColors.appMainColor))),
                      child: Padding(
                        padding: const EdgeInsets.only(left: 8.0, right: 8.0),
                        child: Center(
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Text(
                                "Proceed",
                                style: styleElements
                                    .subtitle2ThemeScalable(context)
                                    .copyWith(
                                        color:
                                            HexColor(AppColors.appColorWhite),
                                        fontSize: 14),
                                textAlign: TextAlign.left,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                    // decoration: BoxDecoration(
                    //   borderRadius: BorderRadius.circular(24.0),
                    //   color: HexColor(AppColors.appMainColor),
                    //   boxShadow: [CommonComponents().getShadowforBox()],
                    // ),
                  ),
                ),
              )
            ],
          ),
        ));
  }
}
