import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/common_button.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import '../../NetworkFile/calls.dart';
import '../../models/base_res.dart';
import '../../notifications/address_notification.dart';
import '../../utils/basicInfo.dart';
import '../../utils/common_helpdesk_sheet.dart';
import '../cubits/cubit_main.dart';
import '../rewards_module/pages/app_account_detail_add.dart';
import '../rewards_module/pages/bank_account_details_page.dart';
import '../rewards_module/pages/business_location_page.dart';

class HeaderWidget extends StatefulWidget {
  final String baseUrl;
  final String? avatar;
  final String? title;
  final String? subTitle;
  final Function refreshCallBack;

  HeaderWidget(
      {this.avatar,
      this.title,
      this.subTitle,
      required this.baseUrl,
      required this.refreshCallBack});

  @override
  HeaderWidgetState createState() => HeaderWidgetState();
}

class HeaderWidgetState extends State<HeaderWidget> {
  BasicInfo? basicInfo = BasicInfo();

  TextStyleElements? styleElements;

  @override
  Widget build(BuildContext context) {
    ScreenUtil.init(context);
    styleElements = TextStyleElements(context);
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        GestureDetector(
          onTap: () {},
          child: Container(
            margin: EdgeInsets.all(4),
            height: 60,
            width: 60,
            child: Align(
                alignment: Alignment.topLeft,
                child: CircleAvatar(
                    key: UniqueKey(),
                    foregroundColor: HexColor(AppColors.appColorWhite),
                    backgroundColor: HexColor(AppColors.appColorWhite),
                    backgroundImage: NetworkImage(
                        basicInfo!.baseUrl + (basicInfo!.userImage)),
                    radius: 52)),
          ),
        ),
        Expanded(
          child: GestureDetector(
            onTap: () {},
            child: Container(
              child: Padding(
                padding: const EdgeInsets.only(left: 12.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Align(
                      alignment: Alignment.topLeft,
                      child: Container(
                          child: Row(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Flexible(
                            child: Text(
                              widget.title ?? "-",
                              maxLines: 2,
                              overflow: TextOverflow.ellipsis,
                              style: styleElements!
                                  .headline6ThemeScalable(context)
                                  .copyWith(
                                      fontWeight: FontWeight.bold,
                                      color:
                                          HexColor(AppColors.appColorBlack85)),
                            ),
                          ),
                        ],
                      )),
                    ),
                    GestureDetector(
                      behavior: HitTestBehavior.translucent,
                      onTap: null,
                      child: Align(
                          alignment: Alignment.bottomLeft,
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Flexible(
                                child: Text(widget.subTitle ?? "-",
                                    maxLines: 2,
                                    overflow: TextOverflow.ellipsis,
                                    style: styleElements!
                                        .captionThemeScalable(context)),
                              )
                            ],
                          )),
                    ),
                    SizedBox(
                      height: 4,
                    )
                  ],
                ),
              ),
            ),
          ),
        ),
        Center(
          child: Container(
              child: AppButtonCommon(
                  padding: const EdgeInsets.all(0),
                  onPressed: () async {
                    showModalBottomSheet(
                      context: context,
                      isScrollControlled: true,
                      shape: const RoundedRectangleBorder(
                          borderRadius: BorderRadius.only(
                              topLeft: Radius.circular(12),
                              topRight: Radius.circular(12))),
                      builder: (BuildContext context) {
                        TextStyleElements styleElements =
                            TextStyleElements(context);
                        return Wrap(
                          children: [
                            CommonHelpdeskSheet(
                              child: BlocProvider(
                                create: (_) => CubitMain(),
                                child: Column(
                                  children: [
                                    ListTile(
                                      title: Text("Edit Address",
                                          style: styleElements
                                              .headline6ThemeScalableW(
                                                  context)),
                                      leading: Icon(Icons.food_bank_outlined),
                                      onTap: () {
                                        Navigator.pop(context);
                                        Navigator.push(
                                            context,
                                            MaterialPageRoute(
                                              builder: (context) =>
                                                  BlocProvider(
                                                create: (_) => CubitMain(),
                                                child:
                                                    BusinessLocationAddressPage(
                                                        widget.refreshCallBack),
                                              ),
                                            ));
                                      },
                                    ),

                                    ListTile(
                                      title: Text("Edit GST details",
                                          style: styleElements
                                              .headline6ThemeScalableW(
                                                  context)),
                                      leading: Icon(Icons.follow_the_signs),
                                      onTap: () {
                                        final body = jsonEncode({
                                          "business_id": basicInfo!.businessId,
                                          "person_id": basicInfo!.userId,
                                          "detail_type": "business",
                                          "owner_id": basicInfo!.userId
                                        });
                                        Calls()
                                            .call(body, context,
                                                basicInfo!.userProfile)
                                            .then((value) async {
                                          var res =
                                              BaseResponses.fromJson(value);
                                          if (res.statusCode ==
                                              basicInfo!.statusCode) {
                                            var val =
                                                res.rows!.firstWhere((element) {
                                              return element.cardName ==
                                                  "AccountingInfoCard";
                                            });
                                            Navigator.pop(context);
                                            Navigator.push(
                                                context,
                                                MaterialPageRoute(
                                                  builder: (context) =>
                                                      AddAccountDetails(
                                                    refreshCallBack:
                                                        widget.refreshCallBack,
                                                    data: val,
                                                  ),
                                                ));
                                          }
                                        });
                                      },
                                    )
                                  ],
                                ),
                              ),
                            ),
                          ],
                        );
                      },
                    );
                  },
                  color: HexColor(AppColors.appMainColor),
                  child: Text(
                    AppLocalizationsRewards.of(context)!.translate('edit'),
                    style: styleElements!
                        .headline6ThemeScalableW(context)
                        .copyWith(color: HexColor(AppColors.blueApp)),
                  ))),
        ),
        Container()
      ],
    );
  }
}
