import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import '../../models/cash_coin_resppnse.dart';
import '../../models/enums/payments_enums.dart';

class InvoiceCard extends StatefulWidget {
  final String? avatar;
  final String? title;
  final String? subTitle;
  final Rows data;
  final String type;

  InvoiceCard(
      {this.avatar,
      this.title,
      this.subTitle,
      required this.data,
      required this.type});

  @override
  InvoiceCardState createState() => InvoiceCardState();
}

class InvoiceCardState extends State<InvoiceCard> {
  final DateFormat formatter = DateFormat('dd-MM-yyyy');
  TextStyleElements? styleElements;
  late String formatted;

  @override
  void initState() {
    var dateTime = DateTime.now();
    formatted = formatter.format(dateTime);

    super.initState();
  }

  String calculateTaxFromPercentage(num serviceCharge, num percentage) {
    var amount = (percentage * serviceCharge) / 100;

    return amount.toStringAsFixed(2).toString();
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisSize: MainAxisSize.min,
      children: [
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "Order date",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!.subtitle2ThemeScalable(context),
              ),
              Text(formatted,
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.headline6ThemeScalableW(context)),
            ],
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 4.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                widget.type == PaymentEnums.cash.type
                    ? AppLocalizationsRewards.of(context)!.translate("casd_amt")
                    : (widget.subTitle!) + " recharge",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!.subtitle1ThemeScalable(context),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                          .translate("rupee_symbol") +
                      (widget.type == PaymentEnums.cash.type
                          ? (widget.data.cashOrCoins!)
                              .toStringAsFixed(2)
                              .toString()
                          : widget.data.serviceChargeAmount!
                              .toStringAsFixed(2)
                              .toString()),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.bodyText2ThemeScalable(context)),
            ],
          ),
        ),
        Visibility(
          visible: widget.type == PaymentEnums.cash.type,
          child: Padding(
            padding: const EdgeInsets.only(top: 8.0, bottom: 4.0),
            child: Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Text(
                  AppLocalizationsRewards.of(context)!.translate('cash_service',
                      arguments: {
                        "perc": widget.data.taxDetails!.serviceCharge.toString()
                      }),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.subtitle1ThemeScalable(context),
                ),
                Text(
                    AppLocalizationsRewards.of(context)!
                            .translate("rupee_symbol") +
                        widget.data.serviceChargeAmount!
                            .toStringAsFixed(2)
                            .toString(),
                    maxLines: 2,
                    overflow: TextOverflow.ellipsis,
                    style: styleElements!.bodyText2ThemeScalable(context)),
              ],
            ),
          ),
        ),
        Visibility(
          visible: widget.type == PaymentEnums.coins.type,
          child: Padding(
            padding: const EdgeInsets.only(top: 4.0, bottom: 8.0),
            child: Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Text(
                  'Qty: ' + widget.data.cashOrCoins.toString(),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.bodyText2ThemeScalable(context),
                ),
              ],
            ),
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "Sub Total",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!
                    .subtitle2ThemeScalable(context)
                    .copyWith(fontWeight: FontWeight.bold),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                                  .translate("rupee_symbol") +
                        (      widget.type ==
                          PaymentEnums.cash.type
                      ? (widget.data.cashOrCoins! +
                              widget.data.serviceChargeAmount!)
                          .toStringAsFixed(2)
                          .toString()
                      : widget.data.serviceChargeAmount!
                          .toStringAsFixed(2)
                          .toString()),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!
                      .headline6ThemeScalableW(context)
                      .copyWith(fontWeight: FontWeight.bold)),
            ],
          ),
        ),
        const Divider(
          height: 2,
          thickness: 0.5,
          indent: 0,
          endIndent: 0,
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "Tax",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!
                    .subtitle1ThemeScalable(context)
                    .copyWith(fontWeight: FontWeight.bold),
              ),
            ],
          ),
        ),
        Visibility(
          visible: widget.data.taxDetails!.igst != null &&
              widget.data.taxDetails!.igst != 0,
          child: Padding(
            padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
            child: Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Text(
                  "IGST",
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.bodyText2ThemeScalable(context),
                ),
                Text(
                    AppLocalizationsRewards.of(context)!
                            .translate("rupee_symbol") +
                        (widget.data.serviceChargeAmount != null &&
                                widget.data.taxDetails!.igst != null
                            ? calculateTaxFromPercentage(
                                widget.data.serviceChargeAmount!,
                                widget.data.taxDetails!.igst!)
                            : "--"),
                    maxLines: 2,
                    overflow: TextOverflow.ellipsis,
                    style: styleElements!.bodyText2ThemeScalable(context)),
              ],
            ),
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "SGST",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!.bodyText2ThemeScalable(context),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                          .translate("rupee_symbol") +
                      (widget.data.serviceChargeAmount != null &&
                              widget.data.taxDetails!.sgst != null
                          ? calculateTaxFromPercentage(
                              widget.data.serviceChargeAmount!,
                              widget.data.taxDetails!.sgst!)
                          : "--"),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.bodyText2ThemeScalable(context)),
            ],
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "CGST",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!.bodyText2ThemeScalable(context),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                          .translate("rupee_symbol") +
                      (widget.data.serviceChargeAmount != null &&
                              widget.data.taxDetails!.cgst != null
                          ? calculateTaxFromPercentage(
                              widget.data.serviceChargeAmount!,
                              widget.data.taxDetails!.cgst!)
                          : "--"),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!.bodyText2ThemeScalable(context)),
            ],
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "Total Tax",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!
                    .subtitle2ThemeScalable(context)
                    .copyWith(fontWeight: FontWeight.bold),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                          .translate("rupee_symbol") +
                      widget.data.taxAmount!.toStringAsFixed(2).toString(),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!
                      .headline6ThemeScalableW(context)
                      .copyWith(fontWeight: FontWeight.bold)),
            ],
          ),
        ),
        const Divider(
          height: 2,
          thickness: 0.5,
          indent: 0,
          endIndent: 0,
        ),
        Padding(
          padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Text(
                "Grand total",
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
                style: styleElements!
                    .subtitle2ThemeScalable(context)
                    .copyWith(fontWeight: FontWeight.bold),
              ),
              Text(
                  AppLocalizationsRewards.of(context)!
                          .translate("rupee_symbol") +
                      widget.data.payableAmount!.toStringAsFixed(2).toString(),
                  maxLines: 2,
                  overflow: TextOverflow.ellipsis,
                  style: styleElements!
                      .headline6ThemeScalableW(context)
                      .copyWith(fontWeight: FontWeight.bold)),
            ],
          ),
        ),
        const Divider(
          height: 2,
          thickness: 0.5,
          indent: 0,
          endIndent: 0,
        ),
      ],
    );
  }
}
