import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';
import 'package:loyalty_management_tool/utils/toast_builder.dart';

class LoyaltyBenifits extends StatefulWidget {
  final String title;
  final bool isBothValueRequired;
  final String subTitle;
  final String firstItemHint;
  final String firstItemTag;

  final Widget firstItemIcon;
  final Widget secondItemIcon;
  final String secondItemHint;
  final String secondItemTag;
  final RewardsItem? item;

  final num? giftCash;
  final num? giftCoin;
  final num? redemptionPercentage;
  final num? redemptionCoins;
  final Function(num cash, num coin) nextCallBack;

  const LoyaltyBenifits(
      {Key? key,
      this.giftCash,
      this.giftCoin,
      this.redemptionPercentage,
      this.redemptionCoins,
      required this.title,
      this.isBothValueRequired = false,
      required this.subTitle,
      this.item,
      required this.nextCallBack,
      required this.secondItemIcon,
      required this.firstItemIcon,
      required this.firstItemHint,
      required this.firstItemTag,
      required this.secondItemHint,
      required this.secondItemTag})
      : super(key: key);

  @override
  LoyaltyBenifitsState createState() => LoyaltyBenifitsState();
}

class LoyaltyBenifitsState extends State<LoyaltyBenifits> {
  GlobalKey<FormState> formKey = GlobalKey();
  final coinsController = TextEditingController();
  final cashController = TextEditingController();

  late TextStyleElements styleElements;

  @override
  initState() {
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      if (widget.item != null) {
        if (widget.title ==
            AppLocalizationsRewards.of(context)!.translate("set_l_benefits")) {
          if (widget.item!.incentiveCoins != null) {
            coinsController.text = widget.item!.incentiveCoins.toString();
          }
          if (widget.item!.incentiveCash != null) {
            cashController.text = widget.item!.incentiveCash.toString();
          }
        } else if (widget.title ==
            AppLocalizationsRewards.of(context)!.translate("dis_red")) {
          if (widget
                  .item!.redemptionDetails!.couponCoinsForDiscountPercentage !=
              null) {
            coinsController.text = widget
                .item!.redemptionDetails!.couponCoinsForDiscountPercentage
                .toString();
          }
          if (widget.item!.redemptionDetails!.couponDiscountPercentage !=
              null) {
            cashController.text = widget
                .item!.redemptionDetails!.couponDiscountPercentage
                .toString();
          }
        } else {
          if (widget.item!.redemptionDetails!.giftCoinsForCash != null) {
            coinsController.text =
                widget.item!.redemptionDetails!.giftCoinsForCash.toString();
          }
          if (widget.item!.redemptionDetails!.giftCash != null) {
            cashController.text =
                widget.item!.redemptionDetails!.giftCash.toString();
          }
        }
      } else if (widget.giftCash != null && widget.giftCoin != null) {
        coinsController.text = widget.giftCoin.toString();
        cashController.text = widget.giftCash.toString();
      } else if (widget.redemptionPercentage != null &&
          widget.redemptionCoins != null) {
        coinsController.text = widget.redemptionCoins.toString();
        cashController.text = widget.redemptionPercentage.toString();
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    final cash = TextFormField(
      controller: cashController,
      keyboardType: TextInputType.number,
      style: styleElements
          .subtitle1ThemeScalable(context)
          .copyWith(color: HexColor(AppColors.appColorBlack65)),
      decoration: InputDecoration(
        fillColor: Colors.black12,
        filled: true,
        prefixIcon: widget.firstItemIcon,
        hintText: widget.firstItemHint,
        contentPadding: const EdgeInsets.all(16),
        border: UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
        floatingLabelBehavior: FloatingLabelBehavior.auto,
        labelText: widget.firstItemTag,
      ),
    );
    final coins = TextFormField(
      controller: coinsController,
      keyboardType: TextInputType.number,
      style: styleElements
          .subtitle1ThemeScalable(context)
          .copyWith(color: HexColor(AppColors.appColorBlack65)),
      decoration: InputDecoration(
        fillColor: Colors.black12,
        filled: true,
        prefixIcon: widget.secondItemIcon,
        hintText: widget.secondItemHint,
        contentPadding: const EdgeInsets.all(16),
        border: UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
        floatingLabelBehavior: FloatingLabelBehavior.auto,
        labelText: widget.secondItemTag,
      ),
    );

    return Padding(
      padding: const EdgeInsets.all(4.0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Padding(
            padding: const EdgeInsets.only(
                bottom: 30.0, left: 16, right: 16, top: 8),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              //Center Row contents horizontally,
              crossAxisAlignment: CrossAxisAlignment.start,
              //Center Row contents vertically,

              children: [
                Expanded(
                  child: Align(
                    alignment: Alignment.center,
                    child: Container(
                      child: Text(
                        widget.title,
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(fontWeight: FontWeight.bold),
                      ),
                    ),
                  ),
                ),
                Align(
                  alignment: Alignment.centerRight,
                  child: InkWell(
                    onTap: () {
                      if (widget.isBothValueRequired
                          ? cashController.text.isNotEmpty &&
                              coinsController.text.isNotEmpty
                          : cashController.text.isNotEmpty ||
                              coinsController.text.isNotEmpty) {
                        Navigator.pop(context);

                        widget.nextCallBack(
                            cashController.text.isNotEmpty
                                ? num.parse(cashController.text.trim())
                                : 0,
                            coinsController.text.isNotEmpty
                                ? num.parse(coinsController.text)
                                : 0);
                      } else {
                        ToastBuilder().showToast(
                            AppLocalizationsRewards.of(context)!.translate(
                                widget.isBothValueRequired
                                    ? "enter_both"
                                    : "enter_one"),
                            context,
                            HexColor(AppColors.information));
                      }
                    },
                    child: Container(
                      child: Text(
                        AppLocalizationsRewards.of(context)!.translate("next"),
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(
                                fontWeight: FontWeight.bold,
                                color: HexColor(AppColors.blueApp)),
                      ),
                    ),
                  ),
                )
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(12),
            child: Text(
              widget.subTitle,
              textAlign: TextAlign.center,
              style: styleElements.captionThemeScalable(context),
            ),
          ),
          Flexible(
              child: Padding(padding: const EdgeInsets.all(8.0), child: cash)),
          const SizedBox(
            width: 20,
          ),
          Flexible(
              child: Padding(padding: const EdgeInsets.all(8.0), child: coins)),
          SizedBox(
            height: MediaQuery.of(context).viewInsets.bottom,
          )
        ],
      ),
    );
  }
}
