import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/recharge_dialog.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';

import 'package:loyalty_management_tool/models/enums/payments_enums.dart';

import '../../models/setting_view_response.dart';
import '../../utils/app_buttons.dart';
import '../../utils/basicInfo.dart';
import '../../utils/colors.dart';
import '../../utils/common_helpdesk_sheet.dart';
import '../../utils/hexColors.dart';
import '../../utils/localization.dart';
import '../../utils/text_styling.dart';
import '../cubits/cubit_main.dart';
import '../rewards_module/pages/payment_screen.dart';
import '../states/main_state.dart';

// ignore: must_be_immutable
class RechargeWalletCard extends StatelessWidget {
  int? userId;

  String? userName;
  String? userType;
  String textOne;
  String textTwo;
  String textThree;
  String textFour;
  String textFive;
  String textSix;
  String textSeven;
  String textEight;
  int? ownerId;
  String? ownerType;
  Null Function()? callback;
  String? businessId;
  String? imageurl;
  final String type;
  final Function refreshCallBack;

  RechargeWalletCard(
      {Key? key,
      required this.type,
      required this.refreshCallBack,
      required this.textOne,
      required this.textTwo,
      required this.textThree,
      required this.textFour,
      required this.textFive,
      required this.textSix,
      required this.textSeven,
      required this.textEight,
      this.userId,
      this.ownerId,
      this.ownerType,
      this.userName,
      this.imageurl,
      this.businessId,
      required this.callback,
      this.userType})
      : super(key: key);
  late TextStyleElements styleElements;
  BasicInfo? basicInfo = BasicInfo();

  Future<void> dataFetch(context) async {
    BlocProvider.of<CubitMain>(context)
        .getData(getBody(), context, basicInfo!.settingView, getData);
  }

  dynamic getData(dynamic data) {
    return SettingViewResponse.fromJson(data);
  }

  String getBody() {
    return jsonEncode({"business_id": basicInfo!.businessId});
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    dataFetch(context);
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      return Container(
        color: HexColor(AppColors.appColorWhite),
        child: Align(
            alignment: Alignment.centerLeft,
            child: Row(children: <Widget>[
              Padding(
                padding: const EdgeInsets.all(16.0),
                child: GestureDetector(
                    onTap: () {
                      /* Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => CourseDetailPage(
                            id: null,

                            */ /* userType: "staff",*/ /*
                          ),
                        ));*/
                    },
                    child: GestureDetector(
                      behavior: HitTestBehavior.translucent,
                      child: Column(
                        children: <Widget>[
                          Container(
                            margin: EdgeInsets.all(2),
                            child: Text(
                              state is LoadedWithData
                                  ? type == PaymentEnums.cash.type
                                      ? state.data.minCash.toStringAsFixed(2).toString()
                                      : state.data.minCoins.toStringAsFixed(2).toString()
                                  : "--",
                              overflow: TextOverflow.ellipsis,
                              style: styleElements
                                  .subtitle2ThemeScalable(context)
                                  .copyWith(
                                      fontWeight: FontWeight.bold,
                                      color:
                                          HexColor(AppColors.appColorBlack85)),
                              textAlign: TextAlign.left,
                            ),
                          ),
                          Container(
                            margin: const EdgeInsets.only(
                                bottom: 2, left: 2, right: 2),
                            child: Text(
                              textOne,
                              overflow: TextOverflow.ellipsis,
                              style:
                                  styleElements.captionThemeScalable(context),
                              textAlign: TextAlign.left,
                            ),
                          ),
                        ],
                      ),
                      onTap: () async {},
                    )),
              ),
              Visibility(
                visible: type == PaymentEnums.coins.type,
                child: Padding(
                  padding: const EdgeInsets.all(16.0),
                  child: GestureDetector(
                      behavior: HitTestBehavior.translucent,
                      onTap: () async {},
                      child: Column(
                        children: <Widget>[
                          Container(
                            margin: EdgeInsets.all(2),
                            child: Text(
                              "100",
                              overflow: TextOverflow.ellipsis,
                              style: styleElements
                                  .subtitle2ThemeScalable(context)
                                  .copyWith(
                                      fontWeight: FontWeight.bold,
                                      color:
                                          HexColor(AppColors.appColorBlack85)),
                              textAlign: TextAlign.left,
                            ),
                          ),
                          Container(
                            margin: const EdgeInsets.only(
                                bottom: 2, left: 2, right: 2),
                            child: Text(
                              "For ₹ " + calculateTaxFromPercentage(100, 2),
                              overflow: TextOverflow.ellipsis,
                              style:
                                  styleElements.captionThemeScalable(context),
                              textAlign: TextAlign.left,
                            ),
                          ),
                        ],
                      )),
                ),
              ),
              Expanded(
                  child: Align(
                alignment: Alignment.centerRight,
                child: appElevatedButton(
                    elevation: 0,
                    shape: StadiumBorder(
                        side: BorderSide(
                            color: HexColor(AppColors.appColorWhite))),
                    onPressed: () {
                      showModalBottomSheet(
                        context: context,
                        isScrollControlled: true,
                        shape: const RoundedRectangleBorder(
                            borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(12),
                                topRight: Radius.circular(12))),
                        builder: (BuildContext context) {
                          return Wrap(
                            children: [
                              CommonHelpdeskSheet(
                                child: BlocProvider(
                                  create: (_) => CubitMain(),
                                  child: RechargeDialog(
                                    type: type,
                                    refreshCallBack: refreshCallBack,
                                    firstItemHint:
                                        AppLocalizationsRewards.of(context)!
                                            .translate(
                                                type == PaymentEnums.cash.type
                                                    ? 'cash'
                                                    : 'coins'),
                                    firstItemTag:
                                        AppLocalizationsRewards.of(context)!
                                            .translate(
                                                type == PaymentEnums.cash.type
                                                    ? 'cas_amount'
                                                    : "c_amount"),
                                    firstItemIcon: Icon(
                                      Icons.currency_bitcoin,
                                      color: HexColor(AppColors.appMainColor),
                                    ),
                                    title: AppLocalizationsRewards.of(context)!
                                        .translate(
                                            type == PaymentEnums.cash.type
                                                ? "recharge_wallet"
                                                : 'recharge_coin'),
                                    nextCallBack: (data) {
                                      Navigator.pop(context);
                                      Navigator.push(
                                          context,
                                          MaterialPageRoute(
                                            builder: (context) => BlocProvider(
                                              create: (_) => CubitMain(),
                                              child: PaymentPage(
                                                  type: type,
                                                  refreshCallBack:
                                                      refreshCallBack,
                                                  data: data,
                                                  title: AppLocalizationsRewards
                                                          .of(context)!
                                                      .translate(type ==
                                                              PaymentEnums
                                                                  .cash.type
                                                          ? "cash_wallet"
                                                          : 'coin_wallet')),
                                            ),
                                          ));
                                    },
                                    subTitle:
                                        AppLocalizationsRewards.of(context)!
                                            .translate(
                                                type == PaymentEnums.cash.type
                                                    ? "r_des_wallet"
                                                    : 'r_des'),
                                  ),
                                ),
                              ),
                            ],
                          );
                        },
                      );
                    },
                    child: Text(
                      "Recharge",
                      style: styleElements
                          .captionThemeScalable(context)
                          .copyWith(color: HexColor(AppColors.appColorWhite)),
                    ),
                    color: HexColor(AppColors.appMainColor)),
              )),
            ])),
      );
    });
  }

  String calculateTaxFromPercentage(num serviceCharge, num percentage) {
    var amount = (percentage * serviceCharge) / 100;

    return amount.toStringAsFixed(2).toString();
  }

  void update(updateFollowers) {
    callback!();
  }
}
