import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/postcardmedia.dart';
import 'package:loyalty_management_tool/bloc/cubits/cubit_main.dart';
import 'package:loyalty_management_tool/bloc/rewards_module/pages/rewardsDetailPage.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';

import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import 'PostCardFooter.dart';

class RewardsWidget extends StatefulWidget {
  final RewardsItem item;
  final Function(RewardsItem? item) editCallBack;
  final Function(int? id) deleteCallBack;
  final bool isDetailPage;

  const RewardsWidget(
      {Key? key,
      required this.item,
      required this.editCallBack,
      this.isDetailPage = false,
      required this.deleteCallBack})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => RewardsWidgetState();
}

class RewardsWidgetState extends State<RewardsWidget> {
  late TextStyleElements styleElements;

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return appCard(
      onTap: () {
        if (!widget.isDetailPage) {
          Navigator.push(
              context,
              MaterialPageRoute(
                builder: (context) => BlocProvider(
                    create: (_) => CubitMain(),
                    child: RewardsDetailPage(
                      item: widget.item,
                      key: null,
                      deleteCallBack: widget.deleteCallBack,
                      editCallBack: widget.editCallBack,
                    )),
              ));
        }
      },
      padding: const EdgeInsets.all(0),
      child: Column(
        children: [
          Visibility(
            visible: widget.item.mediaUrls != null &&
                widget.item.mediaUrls!.isNotEmpty,
            child: PostCardMedia(
              postType: "feed",
              data: widget.item,
              mediaList: widget.item.mediaUrls != null &&
                      widget.item.mediaUrls!.isNotEmpty
                  ? widget.item.mediaUrls
                  : [],
              pagePosition: 0,
              onItemClick: () {},
              editCallBack: widget.editCallBack,
              deleteCallBack: widget.deleteCallBack,
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(
                left: 16, right: 16.0, top: 8.0, bottom: 16.0),
            child: PostCardFooter(
              postOwnerTypeId: widget.item.businessId,
              postId: widget.item.id,
              isRewards: true,
              isDetailPage: widget.isDetailPage,
              postType: "feed",
              data: widget.item,
            ),
          )
        ],
      ),
    );
  }
}
