import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../models/couponheader.dart';
import '../../models/couponlisting.dart';
import '../../models/enums/payments_enums.dart';
import '../../utils/basicInfo.dart';
import '../../utils/colors.dart';
import '../../utils/hexColors.dart';
import '../../utils/text_styling.dart';
import '../cubits/cubit_main.dart';
import '../states/main_state.dart';
import '../states/states.dart';

// ignore: must_be_immutable
class UserPopularityCard extends StatelessWidget {
  int? userId;
  String type;
  String? userName;
  String? userType;
  String textOne;
  String textTwo;
  String textThree;
  String textFour;
  String textFive;
  String textSix;
  String textSeven;
  String textEight;
  int? ownerId;
  String? ownerType;
  Null Function()? callback;
  String? businessId;
  String? imageurl;

  UserPopularityCard(
      {Key? key,
      required this.type,
      required this.textOne,
      required this.textTwo,
      required this.textThree,
      required this.textFour,
      required this.textFive,
      required this.textSix,
      required this.textSeven,
      required this.textEight,
      this.userId,
      this.ownerId,
      this.ownerType,
      this.userName,
      this.imageurl,
      this.businessId,
      required this.callback,
      this.userType})
      : super(key: key);
  late TextStyleElements styleElements;

  BasicInfo basicInfo = BasicInfo();

  Future<void> dataFetch(context) async {
    BlocProvider.of<CubitMain>(context)
        .getData(getBody(), context, basicInfo.couponDetails, getData);
  }

  dynamic getData(dynamic data) {
    return CouponHeaderResponse.fromJson(data);
  }

  String getBody() {
    CouponListRequest payload = CouponListRequest();
    payload.personId = basicInfo.userId.toString();
    payload.ledgerType = type == PaymentEnums.cash.type
        ? PaymentEnums.cash.type
        : PaymentEnums.coins.type;
    payload.businessId = basicInfo.businessId.toString();
    payload.objectType = ObjectType.business.type;

    return jsonEncode(payload);
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    dataFetch(context);
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      return Container(
        color: HexColor(AppColors.appColorWhite),
        child: Align(
            alignment: Alignment.centerLeft,
            child: userType == "business"
                ? Row(children: <Widget>[
                    Expanded(
                      child: GestureDetector(
                          onTap: () {},
                          child: GestureDetector(
                            behavior: HitTestBehavior.translucent,
                            child: Column(
                              children: <Widget>[
                                Container(
                                  margin: EdgeInsets.all(2.h),
                                  child: Text(
                                    state is LoadedWithData
                                        ? state.data.currentBalance
                                        : "--",
                                    overflow: TextOverflow.ellipsis,
                                    style: styleElements
                                        .subtitle2ThemeScalable(context)
                                        .copyWith(
                                            fontWeight: FontWeight.bold,
                                            color: HexColor(
                                                AppColors.appColorBlack85)),
                                    textAlign: TextAlign.left,
                                  ),
                                ),
                                Container(
                                  margin: EdgeInsets.only(
                                      bottom: 2.h, left: 2.h, right: 2.h),
                                  child: Text(
                                    textOne,
                                    overflow: TextOverflow.ellipsis,
                                    style: styleElements
                                        .captionThemeScalable(context),
                                    textAlign: TextAlign.left,
                                  ),
                                ),
                              ],
                            ),
                            onTap: () async {},
                          )),
                    ),
                    Expanded(
                      child: GestureDetector(
                          behavior: HitTestBehavior.translucent,
                          onTap: () async {},
                          child: Column(
                            children: <Widget>[
                              Container(
                                margin: EdgeInsets.all(2.h),
                                child: Text(
                                  textSix,
                                  overflow: TextOverflow.ellipsis,
                                  style: styleElements
                                      .subtitle2ThemeScalable(context)
                                      .copyWith(
                                          fontWeight: FontWeight.bold,
                                          color: HexColor(
                                              AppColors.appColorBlack85)),
                                  textAlign: TextAlign.left,
                                ),
                              ),
                              Container(
                                margin: EdgeInsets.only(
                                    bottom: 2.h, left: 2.h, right: 2.h),
                                child: Text(
                                  textTwo,
                                  overflow: TextOverflow.ellipsis,
                                  style: styleElements
                                      .captionThemeScalable(context),
                                  textAlign: TextAlign.left,
                                ),
                              ),
                            ],
                          )),
                    )
                  ])
                : Row(children: <Widget>[
                    Expanded(
                      child: GestureDetector(
                          onTap: () {
                            /* Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => CourseDetailPage(
                            id: null,

                            */ /* userType: "staff",*/ /*
                          ),
                        ));*/
                          },
                          child: GestureDetector(
                            behavior: HitTestBehavior.translucent,
                            child: Column(
                              children: <Widget>[
                                Container(
                                  margin: EdgeInsets.all(2.h),
                                  child: Text(
                                    state is LoadedWithData
                                        ? state.data.currentBalance
                                        : "--",
                                    overflow: TextOverflow.ellipsis,
                                    style: styleElements
                                        .subtitle2ThemeScalable(context)
                                        .copyWith(
                                            fontWeight: FontWeight.bold,
                                            color: HexColor(
                                                AppColors.appColorBlack85)),
                                    textAlign: TextAlign.left,
                                  ),
                                ),
                                Container(
                                  margin: EdgeInsets.only(
                                      bottom: 2.h, left: 2.h, right: 2.h),
                                  child: Text(
                                    textOne,
                                    overflow: TextOverflow.ellipsis,
                                    style: styleElements
                                        .captionThemeScalable(context),
                                    textAlign: TextAlign.left,
                                  ),
                                ),
                              ],
                            ),
                            onTap: () async {},
                          )),
                    ),
                    Expanded(
                      child: GestureDetector(
                          behavior: HitTestBehavior.translucent,
                          onTap: () async {},
                          child: Column(
                            children: <Widget>[
                              Container(
                                margin: EdgeInsets.all(2.h),
                                child: Text(
                                  textSix,
                                  overflow: TextOverflow.ellipsis,
                                  style: styleElements
                                      .subtitle2ThemeScalable(context)
                                      .copyWith(
                                          fontWeight: FontWeight.bold,
                                          color: HexColor(
                                              AppColors.appColorBlack85)),
                                  textAlign: TextAlign.left,
                                ),
                              ),
                              Container(
                                margin: EdgeInsets.only(
                                    bottom: 2.h, left: 2.h, right: 2.h),
                                child: Text(
                                  textTwo,
                                  overflow: TextOverflow.ellipsis,
                                  style: styleElements
                                      .captionThemeScalable(context),
                                  textAlign: TextAlign.left,
                                ),
                              ),
                            ],
                          )),
                    ),
                    Expanded(
                      child: GestureDetector(
                        behavior: HitTestBehavior.translucent,
                        onTap: () {},
                        child: Column(
                          children: <Widget>[
                            Container(
                              margin: EdgeInsets.all(2.h),
                              child: Text(
                                state is LoadedWithData
                                    ? state.data.status
                                    : "--",
                                overflow: TextOverflow.ellipsis,
                                style: styleElements
                                    .subtitle2ThemeScalable(context)
                                    .copyWith(
                                        fontWeight: FontWeight.bold,
                                        color: HexColor(
                                            AppColors.appColorBlack85)),
                                textAlign: TextAlign.left,
                              ),
                            ),
                            Container(
                              margin: EdgeInsets.only(
                                  bottom: 2.h, left: 2.h, right: 2.h),
                              child: Text(
                                textThree,
                                overflow: TextOverflow.ellipsis,
                                style:
                                    styleElements.captionThemeScalable(context),
                                textAlign: TextAlign.left,
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                    Expanded(
                        child: GestureDetector(
                      behavior: HitTestBehavior.translucent,
                      onTap: () {},
                      child: Column(
                        children: <Widget>[
                          Container(
                            margin: EdgeInsets.all(2.h),
                            child: Text(
                              state is LoadedWithData
                                  ? state.data.isPostPaid
                                      ? "Post Paid"
                                      : "Pre-Paid"
                                  : "--",
                              overflow: TextOverflow.ellipsis,
                              style: styleElements
                                  .subtitle2ThemeScalable(context)
                                  .copyWith(
                                      fontWeight: FontWeight.bold,
                                      color:
                                          HexColor(AppColors.appColorBlack85)),
                              textAlign: TextAlign.left,
                            ),
                          ),
                          Container(
                            margin: EdgeInsets.only(
                                bottom: 2.h, left: 2.h, right: 2.h),
                            child: Text(
                              textFour,
                              overflow: TextOverflow.ellipsis,
                              style:
                                  styleElements.captionThemeScalable(context),
                              textAlign: TextAlign.left,
                            ),
                          ),
                        ],
                      ),
                    )),
                  ])),
      );
    });
  }

  void update(updateFollowers) {
    callback!();
  }
}
