import 'package:flutter/cupertino.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../NetworkFile/calls.dart';
import '../../models/cart_view_response.dart';
import '../../models/create_cart_response.dart';
import '../../models/create_order_response.dart';
import '../../models/create_rewards_reponse.dart';
import '../../models/payment_token_response.dart';
import '../../utils/basicInfo.dart';
import '../states/main_state.dart';
import '../states/states.dart';

typedef Item = dynamic Function(dynamic itemData);

class CubitMain extends Cubit<MainState> {
  CubitMain() : super(LoadingState());
  int page = 1;
  BasicInfo? basicInfo = BasicInfo();

  Future<void> createPost(String body, String refresh, BuildContext ctx,
      String url, Item getItem, bool isEdit) async {
    var res = await Calls().call(
        body, ctx, isEdit ? basicInfo!.editReward : basicInfo!.createRewards);
    var data = CreateRewardResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(LoadingState());
      loadPage(refresh, ctx, url, getItem);
    }
  }

  Future<void> deleteRewards(String body, String refresh, BuildContext ctx,
      String url, Item getItem) async {
    var res = await Calls().call(body, ctx, basicInfo!.deleteReward);
    var data = CreateRewardResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(LoadingState());
      loadPage(refresh, ctx, url, getItem);
    }
  }

  Future<void> getData(
      String body, BuildContext ctx, String url, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode) {
      data.rows != null
          ? emit(LoadedWithData(data: data.rows!))
          : emit(Empty());
    } else {
      emit(ErrorState());
    }
  }

  Future<void> getGstNumber(
      String body, BuildContext ctx, String url, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    dynamic oldData;
    dynamic bankData;

    if (state is LoadedWithData) {
      oldData = (state as LoadedWithData).data;
      bankData = (state as LoadedWithData).bankData;
    }

    if (data.statusCode == basicInfo!.statusCode) {
      var val = data.rows!.firstWhere((element) {
        return element.cardName == "AccountingInfoCard";
      });

      data.rows != null
          ? emit(LoadedWithData(
              data: oldData, bankData: bankData, gst: val.textSix ?? "-"))
          : emit(Empty());
    } else {
      emit(ErrorState());
    }
  }

  Future<void> getAccountDetal(
      String body, BuildContext ctx, String url, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    dynamic oldData;

    dynamic gst;
    if (state is LoadedWithData) {
      oldData = (state as LoadedWithData).data;
      gst = (state as LoadedWithData).gst;
    }

    if (data.statusCode == basicInfo!.statusCode) {
      data.rows != null
          ? emit(LoadedWithData(
              data: oldData,
              bankData: data.rows!.accountNumber ?? "",
              gst: gst))
          : emit(Empty());
    } else {
      emit(ErrorState());
    }
  }

  Future<void> getGst(
      String body, BuildContext ctx, String url, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode) {
      data.rows != null
          ? emit(LoadedWithData(data: data.rows!))
          : emit(Empty());
    } else {
      emit(ErrorState());
    }
  }

  Future<void> updateRedemption(String body, String refreshBody,
      BuildContext ctx, String url, String urlRefresh, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode) {
      getData(refreshBody, ctx, urlRefresh, getItem);
    }
  }

  Future<void> getRedemption(
      String body, BuildContext ctx, String url, Item getItem) async {
    var res = await Calls().call(body, ctx, url);
    final currentState = state;
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode) {
      if (currentState is Loaded) {
        emit(Loaded(
            list: currentState.list,
            isLastPage: currentState.isLastPage,
            selectedItem: currentState.selectedItem,
            giftCash: data.rows.giftCash,
            giftCoin: data.rows.giftCoinsForCash,
            redemptionCoins: data.rows.couponCoinsForDiscountPercentage,
            redemptionPercentage: data.rows.couponDiscountPercentage));
      }
    }
  }

  Future<void> updateSelectedItem(bool isSelected, int indexUpdate) async {
    final currentState = state;
    var list = <dynamic>[];
    var isLastPage = false;
    var selectedItem;
    if (currentState is Loaded) {
      list = currentState.list;
      isLastPage = currentState.isLastPage;
      selectedItem = currentState.selectedItem;
      if (selectedItem != null) {
        var index = list.indexWhere((element) => element.isSelected = true);
        if (index != -1) list[index].isSelected = false;
      }

      list[indexUpdate].isSelected = isSelected;

      emit(Loaded(
          list: list,
          isLastPage: isLastPage,
          selectedItem: isSelected ? list[indexUpdate] : null,
          cash: currentState.cash,
          coins: currentState.coins,
          redemptionPercentage: currentState.redemptionPercentage,
          redemptionCoins: currentState.redemptionCoins,
          giftCash: currentState.giftCash,
          giftCoin: currentState.giftCoin,
          startEpoch: currentState.startEpoch,
          endEpoch: currentState.endEpoch));
    }
  }

  Future<void> updateCashCoins(num cash, num coins) async {
    final currentState = state;
    if (currentState is Loaded) {
      emit(Loaded(
          list: currentState.list,
          isLastPage: currentState.isLastPage,
          selectedItem: currentState.selectedItem,
          cash: cash,
          coins: coins,
          redemptionPercentage: currentState.redemptionPercentage,
          redemptionCoins: currentState.redemptionCoins,
          giftCash: currentState.giftCash,
          giftCoin: currentState.giftCoin,
          startEpoch: currentState.startEpoch,
          endEpoch: currentState.endEpoch));
    }
  }

  Future<void> updateDiscountCouponDeatails(num percentage, num coins) async {
    final currentState = state;

    if (currentState is Loaded) {
      emit(Loaded(
          list: currentState.list,
          isLastPage: currentState.isLastPage,
          selectedItem: currentState.selectedItem,
          cash: currentState.cash,
          coins: currentState.coins,
          redemptionPercentage: percentage,
          redemptionCoins: coins,
          giftCash: currentState.giftCash,
          giftCoin: currentState.giftCoin,
          startEpoch: currentState.startEpoch,
          endEpoch: currentState.endEpoch));
    }
  }

  Future<void> updateGiftCashCoins(num cash, num coins) async {
    final currentState = state;

    if (currentState is Loaded) {
      emit(Loaded(
          list: currentState.list,
          isLastPage: currentState.isLastPage,
          selectedItem: currentState.selectedItem,
          cash: currentState.cash,
          coins: currentState.coins,
          redemptionPercentage: currentState.redemptionPercentage,
          redemptionCoins: currentState.redemptionCoins,
          giftCash: cash,
          giftCoin: coins,
          startEpoch: currentState.startEpoch,
          endEpoch: currentState.endEpoch));
    }
  }

  Future<void> updateStartEndTime(num startTime, num? endTime) async {
    final currentState = state;

    if (currentState is Loaded) {
      emit(Loaded(
          list: currentState.list,
          isLastPage: currentState.isLastPage,
          selectedItem: currentState.selectedItem,
          cash: currentState.cash,
          coins: currentState.coins,
          redemptionPercentage: currentState.redemptionPercentage,
          redemptionCoins: currentState.redemptionCoins,
          giftCash: currentState.giftCash,
          giftCoin: currentState.giftCoin,
          startEpoch: startTime,
          endEpoch: endTime));
    }
  }

  Future<void> search(
      String body, BuildContext ctx, String url, Item getItem) async {
    final currentState = state;
    var selectedItem;
    if (currentState is Loaded) {
      selectedItem = currentState.selectedItem;
    }
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode && data.rows != null) {
      if (selectedItem != null) {
        var index =
            data.rows!.indexWhere((element) => element.id == selectedItem.id);
        if (index != -1) data.rows![index].isSelected = true;
      }
      data.rows != null && data.rows!.isNotEmpty
          ? emit(Loaded(
              list: data.rows ?? [],
              isLastPage: data.rows!.length == data.total,
              selectedItem: selectedItem))
          : emit(Empty());
    }
  }

  Future<void> loadPage(
      String body, BuildContext ctx, String url, Item getItem) async {
    if (state is LoadingStatePagination) return;
    final currentState = state;
    var selectedItem;
    var oldList = <dynamic>[];
    if (currentState is Loaded) {
      if (page != 1) {
        oldList = currentState.list;
      } else {
        currentState.list.clear();
      }
      selectedItem = currentState.selectedItem;
    }
    emit(LoadingStatePagination(list: oldList, isFirstFetch: page == 1));
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    if (data.statusCode == basicInfo!.statusCode && data.rows != null) {
      if (selectedItem != null) {
        var index =
            data.rows!.indexWhere((element) => element.id == selectedItem.id);
        if (index != -1) {
          data.rows![index].isSelected = true;
        }
      }

      final posts = (state as LoadingStatePagination).list;
      posts
          .addAll(data.rows != null && data.rows!.isNotEmpty ? data.rows! : []);
      page++;
      posts.isNotEmpty
          ? emit(Loaded(
              list: posts,
              isLastPage: posts.length == data.total,
              selectedItem: selectedItem))
          : emit(Empty());
    } else if (data.statusCode == basicInfo!.statusCodeEmpty) {
      emit(Empty());
    } else {
      emit(Empty());
    }
  }

  Future<void> getCashCoinRechargeInfo(String body, BuildContext ctx,
      String url, Item getItem, num value) async {
    var res = await Calls().call(body, ctx, url);
    var data = getItem(res);
    emit(LoadingStatePagination(list: []));
    if (data.statusCode == basicInfo!.statusCode) {
      data.rows != null
          ? emit(LoadedWithData(data: data.rows!, selectedNum: value))
          : emit(Empty());
    } else {
      emit(ErrorState());
    }
  }

  Future<void> setPreselectedValue(num value) async {
    if (state is LoadedWithData) {
      emit(LoadedWithData(
          data: (state as LoadedWithData).data, selectedNum: value));
    } else {
      emit(LoadedWithData(data: null, selectedNum: value));
    }
  }

  Future<void> createCart(
    String bodyCartCreate,
    String urlCartCreate,
    BuildContext ctx,
  ) async {
    emit(CreatingCart());
    var res = await Calls().call(bodyCartCreate, ctx, urlCartCreate);
    var data = CartCreateResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(CreateCartCallCompleted(data.rows!.id));
    } else {
      emit(CallFailed());
    }
  }

  Future<void> createOrder(String body, BuildContext ctx, String url) async {
    var res = await Calls().call(body, ctx, url);
    var ids;
    if (state is CreateCartCallCompleted) {
      ids = (state as CreateCartCallCompleted).id;
    }
    var data = CreateOrderResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(CreateOrderCompleted(ids, data.rows!.id, data.rows!.orderNumber));
    } else {
      emit(CallFailed());
    }
  }

  Future<void> paymentToken(String body, BuildContext ctx, String url) async {
    var orderNumber;
    var orderId;
    if (state is CreateOrderCompleted) {
      orderNumber = (state as CreateOrderCompleted).orderNumber;
      orderId = (state as CreateOrderCompleted).orderId;
    }
    var res = await Calls().call(body, ctx, url);
    var data = PaymentTokenResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(PaymentTokenReceived(data.rows!.cftoken, orderNumber, orderId));
    } else {
      emit(CallFailed());
    }
  }

  Future<void> paymentStatus(String body, BuildContext ctx, String url) async {
    var res = await Calls().call(body, ctx, url);
    var data = CreateOrderResponse.fromJson(res);
    if (data.statusCode == basicInfo!.statusCode) {
      emit(PaymentStatusUpdated());
    } else {
      emit(CallFailed());
    }
  }

  Future<void> refreshHistoryPage() async {
    emit(RefreshHistoryPage(true));
  }
}
