import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../../NetworkFile/calls.dart';
import '../../../models/bankDetailsPayload.dart';
import '../../../models/bank_details_response.dart';
import '../../../models/base_res.dart';
import '../../../utils/app_buttons.dart';
import '../../../utils/basicInfo.dart';
import '../../../utils/colors.dart';
import '../../../utils/custom_app_bar.dart';
import '../../../utils/customcard.dart';
import '../../../utils/hexColors.dart';
import '../../../utils/someCommonMixins.dart';
import '../../../utils/text_styling.dart';
import '../../../utils/toast_builder.dart';

class BankAccountDetailsPage extends StatefulWidget {
  final CommonCardData? data;
  final Function? callBack;
  final String type;
  final Function refreshCallBack;

  BankAccountDetailsPage(
      {required this.data,
      this.callBack,
      required this.type,
      required this.refreshCallBack});

  @override
  _BankAccountDetailsPage createState() => _BankAccountDetailsPage();
}

class _BankAccountDetailsPage extends State<BankAccountDetailsPage>
    with CommonMixins {
  TextStyleElements? styleElements;
  SharedPreferences prefs = locatorRewards<SharedPreferences>();
  TextEditingController accountNumberController = TextEditingController();
  TextEditingController accountNumberControllerV = TextEditingController();
  TextEditingController ifscCodeController = TextEditingController();
  TextEditingController accountHolderNameController = TextEditingController();
  TextEditingController accountUpi = TextEditingController();
  bool isEdit = false;
  BasicInfo? basicInfo = BasicInfo();

  @override
  void initState() {
    super.initState();
    if (widget.data != null) {
      isEdit = true;
      accountNumberController.text = widget.data!.textTwo ?? "";
      accountNumberControllerV.text = widget.data!.textTwo ?? "";
      ifscCodeController.text = widget.data!.textFour ?? "";
      accountHolderNameController.text = widget.data!.textSix ?? "";
      accountUpi.text = widget.data!.textEight ?? "";
    } else {
      getDetails();
    }
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    final accountNumber = Container(
      padding: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(12),
        color: HexColor(AppColors.appColorBackground),
      ),
      child: TextFormField(
        validator: validateTextField,
        controller: accountNumberController,
        onSaved: (value) {},
        decoration: InputDecoration(
            hintText: "Bank Account Number",
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText: "Enter Bank Account Number"),
      ),
    );
    final accountNumberV = Container(
      padding: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(12),
        color: HexColor(AppColors.appColorBackground),
      ),
      child: TextFormField(
        validator: validateTextField,
        controller: accountNumberControllerV,
        onSaved: (value) {},
        decoration: InputDecoration(
            hintText: "Re-Enter Bank Account Number",
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText: "Re-Enter Bank Account Number"),
      ),
    );
    final ifsc = Container(
      padding: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(12),
        color: HexColor(AppColors.appColorBackground),
      ),
      child: TextFormField(
        validator: validateTextField,
        controller: ifscCodeController,
        onSaved: (value) {},
        inputFormatters: [
          FilteringTextInputFormatter.allow(RegExp("[0-9a-zA-Z]")),
        ],
        decoration: InputDecoration(
            hintText: "IFSC Code",
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText: "Enter IFSC Code"),
      ),
    );
    final holderName = Container(
      padding: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(12),
        color: HexColor(AppColors.appColorBackground),
      ),
      child: TextFormField(
        validator: validateTextField,
        controller: accountHolderNameController,
        onSaved: (value) {},
        decoration: InputDecoration(
            hintText: "Account Holder Name",
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText: "Enter Account Holder Name"),
      ),
    );
    final upi = Container(
      padding: EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(12),
        color: HexColor(AppColors.appColorBackground),
      ),
      child: TextFormField(
        validator: validateTextField,
        controller: accountUpi,
        onSaved: (value) {},
        decoration: InputDecoration(
            hintText: "UPI ID",
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText: "Enter UPI ID"),
      ),
    );
    return SafeArea(
      child: Scaffold(
        appBar: appAppBar().getCustomAppBar(context,
            appBarTitle: AppLocalizationsRewards.of(context)!
                .translate('bank_account_details'), onBackButtonPress: () {
          Navigator.pop(context);
        }),
        body: appListCard(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: accountNumber,
              ),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: accountNumberV,
              ),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: ifsc,
              ),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: holderName,
              ),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: upi,
              )
            ],
          ),
        ),
        bottomNavigationBar: Container(
          height: 72,
          child: Center(
            child: Row(
              children: [
                Spacer(),
                appTextButton(
                  color: HexColor(AppColors.appMainColor),
                  onPressed: () {
                    _submit();
                  },
                  child: Text(
                    AppLocalizationsRewards.of(context)!.translate('save_exit'),
                    style: styleElements!
                        .headline6ThemeScalableW(context)
                        .copyWith(color: HexColor(AppColors.appColorWhite)),
                  ),
                ),
                SizedBox(
                  width: 16,
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  void _submit() {
    if (accountNumberController.text.isNotEmpty) {
      if (accountNumberControllerV.text.isNotEmpty) {
        if (accountNumberController.text.trim() ==
            accountNumberControllerV.text.trim()) {
          if (accountHolderNameController.text.isNotEmpty) {
            if (ifscCodeController.text.isNotEmpty) {
              BankAccountDetails payload = BankAccountDetails();
              payload.ownerType = widget.type;
              payload.ownerId = basicInfo!.businessId;
              payload.accountType = null;
              payload.accountNumber =
                  accountNumberController.text.toString().trim();
              payload.ifscCode = ifscCodeController.text.toString();
              payload.accountHolderName =
                  accountHolderNameController.text.toString();
              payload.upiId = accountUpi.text.toString();
              Calls()
                  .call(jsonEncode(payload), context, basicInfo!.updateBank)
                  .then((value) {
                var response = BaseResponses.fromJson(value);
                if (response.statusCode == basicInfo!.statusCode) {
                  if (widget.callBack != null) {
                    widget.callBack!();
                  }
                  Navigator.pop(context);
                  widget.refreshCallBack();
                }
              });
            } else {
              ToastBuilder().showToast("Please enter IFSC Code", context,
                  HexColor(AppColors.information));
            }
          } else {
            ToastBuilder().showToast("Please enter account holder name",
                context, HexColor(AppColors.information));
          }
        } else {
          ToastBuilder().showToast("Account numbers not matching", context,
              HexColor(AppColors.information));
        }
      } else {
        ToastBuilder().showToast("Please Re-Enter Account Number", context,
            HexColor(AppColors.information));
      }
    } else {
      ToastBuilder().showToast("Please Enter Account Number", context,
          HexColor(AppColors.information));
    }
  }

  void getDetails() {
    var body = jsonEncode(
        {"owner_type": widget.type, "owner_id": basicInfo!.businessId});
    Calls().call(body, context, basicInfo!.bankDetail).then((value) {
      var response = BankAccountDetailsResponse.fromJson(value);
      if (response.statusCode == basicInfo!.statusCode) {
        if (response.rows != null) {
          accountNumberController.text = response.rows!.accountNumber ?? "";
          accountNumberControllerV.text = response.rows!.accountNumber ?? "";
          ifscCodeController.text = response.rows!.ifscCode ?? "";
          accountHolderNameController.text =
              response.rows!.accountHolderName ?? "";
          accountUpi.text = response.rows!.upiId ?? "";
        }
      }
    });
  }
}
