import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:url_launcher/url_launcher.dart';

import '../../../NetworkFile/calls.dart';
import '../../../models/latlogfromaddress.dart';
import '../../../models/locationDataresponse.dart';
import '../../../utils/basicInfo.dart';
import '../../../utils/colors.dart';
import '../../../utils/customcard.dart';
import '../../../utils/debouncer.dart';
import '../../../utils/hexColors.dart';
import '../../../utils/searchBox.dart';
import '../../../utils/text_styling.dart';
import '../../../utils/toast_builder.dart';

const double CAMERA_ZOOM = 13;
const double CAMERA_TILT = 0;
const double CAMERA_BEARING = 30;

class MapPage extends StatefulWidget {
  final bool isEdit;
  final double? lat;
  final double? long;

  @override
  State<StatefulWidget> createState() => MapPageState();

  MapPage({this.isEdit = false, this.lat, this.long});
}

class MapPageState extends State<MapPage> {
  double? lat = 0;
  double? long = 0;

  LatLng SOURCE_LOCATION = LatLng(0, 0);
  BasicInfo? basicInfo = BasicInfo();
  bool isSuggestionsVisible = false;
  bool isItemCLicked = false;
  CameraPosition? initialLocation;
  GoogleMapController? _controller;
  Map<MarkerId, Marker> _markers = <MarkerId, Marker>{};
  int _markerIdCounter = 0;
  List<Predictions>? predictions;
  final _debouncer = Debouncer(600);
  Completer<GoogleMapController> _mapController = Completer();
  late BitmapDescriptor sourceIcon;
  bool isSearching = false;
  GoogleMap? map;
  GlobalKey<SearchBoxState> searchPageKey = GlobalKey();
  TextEditingController controller = TextEditingController();
  TextEditingController controller2 = TextEditingController();
  final GeolocatorPlatform _geolocatorPlatform = GeolocatorPlatform.instance;
  bool isLoading = true;

  @override
  initState() {
    super.initState();

    if (widget.isEdit) {
      if (widget.lat != null &&
          widget.lat != 0 &&
          widget.long != null &&
          widget.long != 0) {
        setState(() {
          lat = widget.lat;
          long = widget.long;
          SOURCE_LOCATION = LatLng(lat!, long!);
          getAddressFromLatLon(lat!, long!);
        });
      } else
        _getCurrentPosition();
    } else {
      setState(() {
        if (lat != null) {
          lat = widget.lat;
          long = widget.long;
          SOURCE_LOCATION = LatLng(lat!, long!);
        } else {
          _getCurrentPosition();
        }
      });
    }
    setSourceAndDestinationIcons();
  }

  Future<void> _getCurrentPosition() async {
    final hasPermission = await _handlePermission();

    if (!hasPermission) {
      ToastBuilder().showToast("Location permission required ", context,
          HexColor(AppColors.information));
    }

    final position = await _geolocatorPlatform.getCurrentPosition();
    lat = position.latitude;
    long = position.longitude;

    if (lat != null) {
      getAddressFromLatLon(lat!, long!);
      setState(() {
        SOURCE_LOCATION = LatLng(lat!, long!);
      });
    }
  }

  Future<bool> _handlePermission() async {
    LocationPermission permission;

    permission = await _geolocatorPlatform.checkPermission();
    if (permission == LocationPermission.denied) {
      permission = await _geolocatorPlatform.requestPermission();
      if (permission == LocationPermission.denied) {
        return false;
      }
    }

    if (permission == LocationPermission.deniedForever) {
      return false;
    }

    return true;
  }

  String _markerIdVal({bool increment = false}) {
    String val = 'marker_id_$_markerIdCounter';
    if (increment) _markerIdCounter++;
    return val;
  }

  void setSourceAndDestinationIcons() async {
    sourceIcon = await BitmapDescriptor.fromAssetImage(
        ImageConfiguration(devicePixelRatio: 2.5),
        'assets/appimages/locations.png');
  }

  void onsearchValueChanged(String text) {
    if (text.length < 3) {
      setState(() {
        isSuggestionsVisible = false;
        isSearching = false;
      });
    } else {
      _debouncer.run(() {
        getSuggestions(text);
      });
      setState(() {
        isSearching = true;
      });
    }
  }

  void getSuggestions(String searchVal) async {
    var apiKey = basicInfo!.googleMapKey;
    final request =
        "https://maps.googleapis.com/maps/api/place/queryautocomplete/json?key=$apiKey&input=$searchVal&types=address";
    Calls().getRequest(context, request).then((value) async {
      if (value != null) {
        var data = LocationsData.fromJson(value);
        print(value);

        setState(() {
          predictions = data.predictions ?? [];
          if (predictions != null && predictions!.isNotEmpty)
            isSuggestionsVisible = true;
        });
      }
    }).catchError((onError) {});
  }

  void getLatLon(String address) async {
    var apiKey = basicInfo!.googleMapKey;
    final request =
        "https://maps.googleapis.com/maps/api/geocode/json?place_id=$address&key=$apiKey";
    Calls().getRequest(context, request).then((value) async {
      if (value != null) {
        var data = LatLongAddress.fromJson(value);
        print(value);

        setState(() {
          controller.text = data.results![0].formattedAddress!;

          setState(() {
            lat = data.results![0].geometry!.location!.lat!;
            long = data.results![0].geometry!.location!.lng!;
          });

          _moveTo(data.results![0].geometry!.location!.lat!,
              data.results![0].geometry!.location!.lng!);
        });
      }
    }).catchError((onError) {});
  }

  void getAddressFromLatLon(double lat, double lon) async {
    setState(() {
      isLoading = true;
    });

    var apiKey = basicInfo!.googleMapKey;
    final request =
        "https://maps.googleapis.com/maps/api/geocode/json?latlng=$lat,$lon&key=$apiKey";
    Calls().getRequest(context, request).then((value) async {
      if (value != null) {
        setState(() {
          isLoading = false;
        });
        var data = LatLongAddress.fromJson(value);
        print(value);

        setState(() {
          controller.text = data.results![0].formattedAddress!;
        });
      }
    }).catchError((onError) {
      setState(() {
        isLoading = false;
      });
    });
  }

  _moveTo(double latitude, double longitude) async {
    await _controller!.animateCamera(
      CameraUpdate.newCameraPosition(
        CameraPosition(
          target: LatLng(latitude, longitude),
          zoom: 17.0,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    TextStyleElements styleElements = TextStyleElements(context);
    if (lat != 0 && long != 0) {
      initialLocation = CameraPosition(
          zoom: CAMERA_ZOOM,
          bearing: CAMERA_BEARING,
          tilt: CAMERA_TILT,
          target: SOURCE_LOCATION);

      map = GoogleMap(
        markers: Set<Marker>.of(_markers.values),
        onMapCreated: _onMapCreated,
        initialCameraPosition: initialLocation!,
        myLocationEnabled: false,
        onCameraMove: (CameraPosition newPosition) async {
          if (_markers.length > 0) {
            MarkerId markerId = MarkerId(_markerIdVal());
            Marker? marker = _markers[markerId];
            Marker updatedMarker = marker!.copyWith(
              positionParam: newPosition.target,
            );

            setState(() {
              _markers[markerId] = updatedMarker;
            });
          }
          _debouncer.run(() async {
            setState(() {
              lat = newPosition.target.latitude;
              long = newPosition.target.longitude;
            });
            getAddressFromLatLon(
                newPosition.target.latitude, newPosition.target.longitude);
          });

          print(newPosition.target.latitude);
        },
      );
      return Scaffold(
        body: Stack(
          children: [
            lat != 0
                ? Container(
                    width: MediaQuery.of(context).size.width,
                    height: MediaQuery.of(context).size.height,
                    child: map)
                : Container(),
            Align(
              alignment: Alignment.topLeft,
              child: Padding(
                padding: const EdgeInsets.only(left: 20.0, top: 50.0),
                child: GestureDetector(
                  behavior: HitTestBehavior.translucent,
                  onTap: () {
                    Navigator.of(context).pop({'result': "update"});
                  },
                  child: Icon(
                    Icons.keyboard_backspace_rounded,
                    size: 20.0,
                    color: HexColor(AppColors.appColorBlack),
                  ),
                ),
              ),
            ),
            Align(
              alignment: Alignment.bottomRight,
              child: Padding(
                padding: const EdgeInsets.only(right: 16.0, bottom: 100.0),
                child: GestureDetector(
                  onTap: () {
                    String googleUrl =
                        'https://www.google.com/maps/search/?api=1&query=$lat,$long';
                    try {
                      launch(googleUrl);
                    } on Exception catch (e) {
                      print(e.toString());
                    }
                  },
                  child: Container(
                    child: Icon(
                      Icons.directions,
                      size: 52,
                      color: HexColor(AppColors.appColorBlack),
                    ),
                  ),
                ),
              ),
            ),
            Align(
              alignment: Alignment.topCenter,
              child: Padding(
                padding: const EdgeInsets.only(top: 80.0),
                child: Column(
                  children: [
                    Visibility(
                      visible: widget.isEdit,
                      child: SearchBox(
                        key: searchPageKey,
                        clearText: InkWell(
                          onTap: () {
                            FocusScopeNode currentFocus =
                                FocusScope.of(context);
                            searchPageKey.currentState!.controller.clear();
                            if (!currentFocus.hasPrimaryFocus) {
                              currentFocus.unfocus();
                            }
                          },
                          child: Container(
                            child: Padding(
                              padding: const EdgeInsets.all(8.0),
                              child: Icon(
                                Icons.cancel_outlined,
                                color: HexColor(AppColors.appMainColor),
                                size: 30,
                              ),
                            ),
                          ),
                        ),
                        onvalueChanged: onsearchValueChanged,
                        hintText: AppLocalizationsRewards.of(context)!
                            .translate('search'),
                      ),
                    ),
                    Visibility(
                      visible: isSearching && isSuggestionsVisible,
                      child: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: appCard(
                          color: HexColor(AppColors.appColorWhite),
                          child: predictions != null && predictions!.isNotEmpty
                              ? SizedBox(
                                  height: 300,
                                  child: ListView.builder(
                                      padding: const EdgeInsets.all(8),
                                      itemCount: predictions!.length,
                                      itemBuilder:
                                          (BuildContext context, int index) {
                                        return InkWell(
                                          onTap: () {
                                            setState(() {
                                              FocusScopeNode currentFocus =
                                                  FocusScope.of(context);
                                              searchPageKey
                                                  .currentState!.controller
                                                  .clear();
                                              if (!currentFocus
                                                  .hasPrimaryFocus) {
                                                currentFocus.unfocus();
                                              }
                                              isSuggestionsVisible = false;
                                              getLatLon(
                                                  predictions![index].placeId!);
                                            });
                                          },
                                          child: ListTile(
                                            title: Text(predictions![index]
                                                .structuredFormatting!
                                                .mainText!),
                                            subtitle: Text(predictions![index]
                                                    .structuredFormatting!
                                                    .secondaryText ??
                                                ""),
                                          ),
                                        );
                                      }),
                                )
                              : Container(),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
            Align(
              alignment: Alignment.bottomCenter,
              child: Container(
                height: 200,
                child: Visibility(
                  visible: !isSuggestionsVisible && widget.isEdit,
                  child: Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: appCard(
                      color: HexColor(AppColors.appColorWhite),
                      child: Stack(
                        children: [
                          ListTile(
                            title: TextField(
                              controller: controller,
                              enabled: false,
                              onChanged: (value) {},
                              maxLines: 3,
                              style: styleElements
                                  .headline6ThemeScalable(context)
                                  .copyWith(
                                      fontWeight: FontWeight.bold,
                                      color:
                                          HexColor(AppColors.appColorBlack85)),
                              decoration: InputDecoration(
                                border: InputBorder.none,
                                hintStyle: styleElements
                                    .headline6ThemeScalable(context)
                                    .copyWith(
                                        fontWeight: FontWeight.bold,
                                        color: HexColor(
                                            AppColors.appColorBlack85)),
                              ),
                            ),
                            subtitle: TextField(
                              controller: controller2,
                              enabled: false,
                              onChanged: (value) {},
                              maxLines: 3,
                              style:
                                  styleElements.subtitle2ThemeScalable(context),
                              decoration: InputDecoration(
                                border: InputBorder.none,
                                hintStyle: styleElements
                                    .subtitle2ThemeScalable(context),
                              ),
                            ),
                          ),
                          Align(
                            alignment: Alignment.bottomRight,
                            child: TextButton(
                                child: Text(
                                  AppLocalizationsRewards.of(context)!
                                      .translate('next'),
                                  style: styleElements
                                      .headline6ThemeScalableW(context)
                                      .copyWith(
                                          color: HexColor(AppColors.blueApp)),
                                ),
                                onPressed: () {
                                  Navigator.of(context).pop({
                                    'address': controller.text.toString(),
                                    'lat': lat,
                                    'lon': long,
                                  });
                                }),
                          ),
                          Visibility(
                            visible: isLoading,
                            child: Align(
                                alignment: Alignment.center,
                                child: Container(
                                    child: CircularProgressIndicator())),
                          )
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      );
    }
    return Container();
  }

  void _onMapCreated(GoogleMapController controller) async {
    _controller = controller;
    _mapController.complete(controller);
    if (lat != 0) {
      MarkerId markerId = MarkerId(_markerIdVal());
      LatLng position = SOURCE_LOCATION;
      Marker marker = Marker(
        markerId: markerId,
        position: position,
        draggable: false,
      );
      setState(() {
        _markers[markerId] = marker;
      });

      Future.delayed(Duration(seconds: 1), () async {
        GoogleMapController controller = await _mapController.future;
        controller.animateCamera(
          CameraUpdate.newCameraPosition(
            CameraPosition(
              target: position,
              zoom: 17.0,
            ),
          ),
        );
      });
    }
  }
}
