import 'dart:convert';
import 'package:intl/intl.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/paginator_list_view.dart';
import 'package:loyalty_management_tool/bloc/cubits/cubit_main.dart';
import 'package:loyalty_management_tool/bloc/states/main_state.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/utils/appAvatar.dart';

import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/empty_widget.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../../models/couponlisting.dart';

class LeadgerHistoryPage extends StatefulWidget {
  String? type;

  bool refreshHistory;

  LeadgerHistoryPage(
      {Key? key, required this.type, required this.refreshHistory})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => LeadgerHistoryPageState();
}

class LeadgerHistoryPageState extends State<LeadgerHistoryPage> {
  final scrollController = ScrollController();
  late TextStyleElements styleElements;
  List<dynamic> list = [];
  bool isLastPage = false;
  bool isActive = true;
  String? searchValue;
  BasicInfo? basicInfo = BasicInfo();
  List<PopupMenuItem> menuList = [];
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  late BuildContext ctx;

  void refresh(String _type) {
    widget.type = _type;
    BlocProvider.of<CubitMain>(context).page = 1;
    BlocProvider.of<CubitMain>(context)
        .loadPage(getBody(), context, basicInfo!.couponLeaderList, getData);
  }

  @override
  void initState() {
    setupScrollController();

    WidgetsBinding.instance!.addPostFrameCallback((_) {
      BlocProvider.of<CubitMain>(context)
          .loadPage(getBody(), context, basicInfo!.couponLeaderList, getData);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    ctx = context;

    styleElements = TextStyleElements(context);

    return Scaffold(
      body: _getBody(),
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);

      if (state is LoadingStatePagination && state.isFirstFetch) {
        return LoadingIndicator();
      }
      bool isLoading = false;
      if (state is LoadingStatePagination) {
        list = state.list;
        isLoading = true;
      } else if (state is Loaded) {
        list = state.list;
        isLastPage = state.isLastPage;
      } else if (state is Empty) {
        return Center(
          child: EmptyWidget(
              message:
                  AppLocalizationsRewards.of(context)!.translate("no_data")),
        );
      }
      return Padding(
        padding: const EdgeInsets.only(top: 8.0),
        child: appCard(
          padding: const EdgeInsets.all(0),
          child: PaginatorListView(
            scrollController: scrollController,
            list: list,
            isLoading: isLoading,
            itemBuilder: getPostWidget,
          ),
        ),
      );
    });
  }

  Widget getPostWidget(value, index) {
    CouponHistoryItem item = value;
    var parsedDate = DateTime.parse(item.createdDate ?? "");
    var selectedDate = DateFormat('dd-MMM-yyyy').format(parsedDate);
    return ListTile(
      leading: Image.asset(
        'packages/loyalty_management_tool/assets/appimages/rech.png',
        width: 36,
        height: 36,
      ),
      title: Text(item.transactionType ?? "-",
          style: styleElements.subtitle2ThemeScalable(context)),
      subtitle: Text(selectedDate,
          style: styleElements.captionThemeScalable(context)),
      trailing: Text(
          item.rewardsCr != null &&
                  item.rewardsCr!.isNotEmpty &&
                  item.rewardsCr != "0.00"
              ? item.rewardsCr ?? "-"
              : item.rewardsDr ?? "-",
          style: styleElements.subtitle2ThemeScalable(context).copyWith(
              color: HexColor(item.rewardsCr != null &&
                      item.rewardsCr!.isNotEmpty &&
                      item.rewardsCr != "0.00"
                  ? AppColors.appColorGreen
                  : AppColors.appColorRed))),
    );
  }

  dynamic getData(dynamic data) {
    return CouponHistoryResponse.fromJson(data);
  }

  String getBody() {
    CouponListRequest request = CouponListRequest();
    request.businessId = basicInfo!.businessId.toString();
    request.objectType = "business";
    request.ledgerType = widget.type;
    request.personId = basicInfo!.userId.toString();
    request.pageNumber = BlocProvider.of<CubitMain>(ctx).page;
    request.pageSize = 10;
    return jsonEncode(request);
  }

  void setupScrollController() {
    scrollController.addListener(() {
      if (scrollController.position.atEdge) {
        if (scrollController.position.pixels != 0) {
          if (!isLastPage) {
            BlocProvider.of<CubitMain>(ctx)
                .loadPage(getBody(), ctx, basicInfo!.couponLeaderList, getData);
          }
        }
      }
    });
  }
}
