import 'dart:convert';
import 'dart:developer';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/common_list_tile.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/date_time_widget.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/loyalty_benifits.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/paginator_list_view.dart';
import 'package:loyalty_management_tool/bloc/cubits/cubit_main.dart';
import 'package:loyalty_management_tool/bloc/rewards_module/pages/postcreatepage.dart';
import 'package:loyalty_management_tool/bloc/rewards_module/pages/set_loyalty_details.dart';
import 'package:loyalty_management_tool/bloc/states/main_state.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/models/LoyaltyTypeList.dart';
import 'package:loyalty_management_tool/models/RedemptionResponse.dart';
import 'package:loyalty_management_tool/models/create_post_payload.dart';
import 'package:loyalty_management_tool/models/postcreate.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:loyalty_management_tool/utils/appAvatar.dart';
import 'package:loyalty_management_tool/utils/app_buttons.dart';
import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/common_button.dart';
import 'package:loyalty_management_tool/utils/common_helpdesk_sheet.dart';
import 'package:loyalty_management_tool/utils/custom_app_bar.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:loyalty_management_tool/utils/resolutionenums.dart';
import 'package:loyalty_management_tool/utils/searchBox.dart';
import 'package:loyalty_management_tool/utils/serviceTypeEnums.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';
import 'package:loyalty_management_tool/utils/toast_builder.dart';
import 'package:shared_preferences/shared_preferences.dart';

class LoyaltyTypePageNew extends StatefulWidget {
  final RewardsItem? item;

  LoyaltyTypePageNew({Key? key, required this.item}) : super(key: key);

  @override
  LoyaltyTypePageNewState createState() => LoyaltyTypePageNewState();
}

class LoyaltyTypePageNewState extends State<LoyaltyTypePageNew> {
  final scrollController = ScrollController();
  late TextStyleElements styleElements;
  List<dynamic> list = [];
  bool isLastPage = false;
  String? searchValue;
  LoyaltyTypeItem? selectedItem;
  BasicInfo? basicInfo = BasicInfo();
  late Loaded loadedSate;
  CreateRewardPayload rewardsData = CreateRewardPayload();
  RedemptionDetails rewardsDatails = RedemptionDetails();
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  late BuildContext ctx;

  void setupScrollController() {
    scrollController.addListener(() async {
      if (scrollController.position.atEdge) {
        if (scrollController.position.pixels != 0) {
          if (!isLastPage) {
            await BlocProvider.of<CubitMain>(ctx)
                .loadPage(getBody(), ctx, basicInfo!.loyaltyTypeList, getData);
          }
        }
      }
    });
  }

  dynamic getDataRed(dynamic data) {
    return RedemptionResponse.fromJson(data);
  }

  dynamic getData(dynamic data) {
    return LoyaltyTypeList.fromJson(data);
  }

  String getBodyRed() {
    return jsonEncode({
      "business_id": basicInfo!.businessId,
    });
  }

  String getBody() {
    return jsonEncode({
      "entity_id": basicInfo!.businessId,
      "search_val": searchValue,
      "page_size": 20,
      "page_number": BlocProvider.of<CubitMain>(ctx).page,
      "app_type": "FOOPWORKS"
    });
  }

  @override
  void initState() {
    setupScrollController();

    WidgetsBinding.instance!.addPostFrameCallback((_) async {
      await BlocProvider.of<CubitMain>(context)
          .loadPage(getBody(), context, basicInfo!.loyaltyTypeList, getData);
      BlocProvider.of<CubitMain>(context).getRedemption(
          getBodyRed(), context, basicInfo!.dafoultRedemption, getDataRed);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    ctx = context;
    styleElements = TextStyleElements(context);

    return Scaffold(
      appBar: appAppBar().getCustomAppBar(
        context,
        appBarTitle:
            AppLocalizationsRewards.of(context)!.translate('select_l_type'),
        actions: [
          appTextButton(
            onPressed: () {
              if (selectedItem != null) {
                showModalBottomSheet(
                  context: context,
                  isScrollControlled: true,
                  shape: const RoundedRectangleBorder(
                      borderRadius: BorderRadius.only(
                          topLeft: Radius.circular(12),
                          topRight: Radius.circular(12))),
                  builder: (BuildContext context) {
                    return Wrap(
                      children: [
                        CommonHelpdeskSheet(
                          child: LoyaltyBenifits(
                            item: widget.item,
                            firstItemHint: AppLocalizationsRewards.of(context)!
                                .translate('coins'),
                            firstItemTag: AppLocalizationsRewards.of(context)!
                                .translate('coin_enter'),
                            secondItemHint: AppLocalizationsRewards.of(context)!
                                .translate('cash'),
                            firstItemIcon: Icon(
                              Icons.flip_camera_android_sharp,
                              color: HexColor(AppColors.appColorRed),
                            ),
                            secondItemTag: AppLocalizationsRewards.of(context)!
                                .translate('cash_incentive'),
                            title: AppLocalizationsRewards.of(context)!
                                .translate("set_l_benefits"),
                            nextCallBack: (cash, coin) {
                              BlocProvider.of<CubitMain>(ctx)
                                  .updateCashCoins(cash, coin);
                              openManageRedemption();
                            },
                            subTitle: AppLocalizationsRewards.of(context)!
                                .translate("incentive_des"),
                            secondItemIcon: Icon(
                              Icons.insert_emoticon_outlined,
                              color: HexColor(AppColors.appColorRed),
                            ),
                          ),
                        ),
                      ],
                    );
                  },
                );
              } else {
                ToastBuilder().showToast(
                    AppLocalizationsRewards.of(context)!
                        .translate("select_type"),
                    context,
                    HexColor(AppColors.information));
              }
            },
            child: Wrap(
              direction: Axis.horizontal,
              crossAxisAlignment: WrapCrossAlignment.center,
              children: [
                const Visibility(
                  visible: false,
                  child: Padding(
                    padding: EdgeInsets.only(right: 8.0),
                    child: SizedBox(
                      height: 15,
                      width: 15,
                      child: CircularProgressIndicator(),
                    ),
                  ),
                ),
                Text(
                  AppLocalizationsRewards.of(context)!.translate("next"),
                  style: styleElements
                      .headline6ThemeScalableW(context)
                      .copyWith(color: HexColor(AppColors.blueApp)),
                ),
                Icon(Icons.keyboard_arrow_right,
                    color: HexColor(AppColors.blueApp))
              ],
            ),
            shape: const CircleBorder(),
          ),
        ],
        onBackButtonPress: () {
          Navigator.of(context).pop();
        },
      ),
      body: Stack(
        children: <Widget>[
          NestedScrollView(
              headerSliverBuilder:
                  (BuildContext context, bool innerBoxIsScrolled) {
                return [
                  SliverToBoxAdapter(
                    child: SearchBox(
                      onvalueChanged: (s) {
                        searchValue = s;
                        BlocProvider.of<CubitMain>(ctx).page = 1;
                        s.isNotEmpty
                            ? BlocProvider.of<CubitMain>(ctx).search(getBody(),
                                ctx, basicInfo!.loyaltyTypeList, getData)
                            : BlocProvider.of<CubitMain>(ctx).loadPage(
                                getBody(),
                                ctx,
                                basicInfo!.loyaltyTypeList,
                                getData);
                      },
                      hintText: AppLocalizationsRewards.of(context)!
                          .translate('search'),
                    ),
                  ),
                ];
              },
              body: _getBody()),
        ],
      ),
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);

      if (state is LoadingStatePagination && state.isFirstFetch) {
        return LoadingIndicator();
      }
      bool isLoading = false;
      if (state is LoadingStatePagination) {
        list = state.list;
        isLoading = true;
      } else if (state is Loaded) {
        loadedSate = state;
        list = state.list;
        isLastPage = state.isLastPage;
        selectedItem = state.selectedItem;

        print("gift cash =" +
            loadedSate.giftCash.toString() +
            "gift coin----" +
            loadedSate.giftCoin.toString());
      }

      return appCard(
          padding: const EdgeInsets.only(left: 0, right: 0, bottom: 60, top: 0),
          child: Padding(
            padding: const EdgeInsets.only(top: 8.0),
            child: PaginatorListView(
              scrollController: scrollController,
              list: list,
              isLoading: isLoading,
              itemBuilder: listItemBuilder,
            ),
          ));
    });
  }

  openManageRedemption() {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return Wrap(
          children: [
            CommonHelpdeskSheet(
              child: LoyaltyBenifits(
                item: widget.item,
                redemptionCoins: loadedSate.redemptionCoins,
                redemptionPercentage: loadedSate.redemptionPercentage,
                isBothValueRequired: true,
                firstItemHint:
                    AppLocalizationsRewards.of(context)!.translate('per_hint'),
                firstItemTag:
                    AppLocalizationsRewards.of(context)!.translate('per_hint'),
                secondItemHint:
                    AppLocalizationsRewards.of(context)!.translate('coins'),
                secondItemTag:
                    AppLocalizationsRewards.of(context)!.translate('en_per'),
                firstItemIcon: Icon(
                  Icons.build,
                  color: HexColor(AppColors.appColorRed),
                ),
                title:
                    AppLocalizationsRewards.of(context)!.translate("dis_red"),
                nextCallBack: (cash, coin) {
                  BlocProvider.of<CubitMain>(ctx)
                      .updateDiscountCouponDeatails(cash, coin);
                  openManageGiftRedemption();
                },
                subTitle:
                    AppLocalizationsRewards.of(context)!.translate("re_des"),
                secondItemIcon: Icon(
                  Icons.gamepad,
                  color: HexColor(AppColors.appColorRed),
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  openManageGiftRedemption() {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return Wrap(
          children: [
            CommonHelpdeskSheet(
              child: LoyaltyBenifits(
                isBothValueRequired: true,
                item: widget.item,
                giftCoin: loadedSate.giftCoin,
                giftCash: loadedSate.giftCash,
                firstItemHint:
                    AppLocalizationsRewards.of(context)!.translate('enter_amt'),
                firstItemTag:
                    AppLocalizationsRewards.of(context)!.translate('enter_amt'),
                secondItemHint:
                    AppLocalizationsRewards.of(context)!.translate('coins'),
                secondItemTag:
                    AppLocalizationsRewards.of(context)!.translate('en_per'),
                firstItemIcon: Icon(
                  Icons.upload_outlined,
                  color: HexColor(AppColors.appColorRed),
                ),
                title:
                    AppLocalizationsRewards.of(context)!.translate("mng_gft"),
                nextCallBack: (percentage, coin) {
                  BlocProvider.of<CubitMain>(ctx)
                      .updateGiftCashCoins(percentage, coin);
                  openManageDateTime();
                },
                subTitle:
                    AppLocalizationsRewards.of(context)!.translate("pg_jj"),
                secondItemIcon: Icon(
                  Icons.youtube_searched_for_rounded,
                  color: HexColor(AppColors.appColorRed),
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  openManageDateTime() {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return Wrap(
          children: [
            CommonHelpdeskSheet(
              child: DateTimeWidget(
                item: widget.item,
                subTitle: AppLocalizationsRewards.of(context)!
                    .translate("program_period_des"),
                title: AppLocalizationsRewards.of(context)!
                    .translate("program_period"),
                nextCallBack: (num stDate, num? edDate) {
                  BlocProvider.of<CubitMain>(ctx)
                      .updateStartEndTime(stDate, edDate);

                  Navigator.of(context)
                      .push(MaterialPageRoute(
                          builder: (context) => PostCreatePage(
                                type: "feed",
                                recipientType: widget.item != null
                                    ? widget.item!.recipientType
                                    : null,
                                recipientDetails: widget.item != null
                                    ? widget.item!.recipientDetails
                                    : null,
                                isWelcomeMessage: true,
                                mediaFromEdit: widget.item != null &&
                                        widget.item!.mediaUrls != null &&
                                        widget.item!.mediaUrls!.isNotEmpty
                                    ? widget.item!.mediaUrls
                                    : null,
                                postId: widget.item != null
                                    ? widget.item!.id
                                    : null,
                                contentData: widget.item != null
                                    ? widget.item!.subtitle
                                    : null,
                                titleLesson: widget.item != null
                                    ? widget.item!.title
                                    : null,
                                isEdit: true,
                              )))
                      .then((value) => {
                            if (value != null && value['payload'] != null)
                              {
                                rewardsDatails.giftCash = loadedSate.giftCash,
                                rewardsDatails.giftCoinsForCash =
                                    loadedSate.giftCoin,
                                rewardsDatails.couponDiscountPercentage =
                                    loadedSate.redemptionPercentage,
                                rewardsDatails
                                        .couponCoinsForDiscountPercentage =
                                    loadedSate.redemptionCoins,
                                rewardsData.standardLoyaltyProgramTypesId =
                                    loadedSate.selectedItem.loyaltyTypeId,
                                rewardsData.startDatetime =
                                    loadedSate.startEpoch,
                                rewardsData.endDatetime = loadedSate.endEpoch,
                                rewardsData.incentiveCash = loadedSate.cash,
                                rewardsData.incentiveCoins = loadedSate.coins,
                                rewardsData.incentiveCurrency = "INR",
                                rewardsData.status = widget.item != null
                                    ? widget.item!.status ?? "active"
                                    : "active",
                                rewardsData.businessId = basicInfo!.businessId,
                                rewardsData.redemptionDetails = rewardsDatails,
                                rewardsData.mediaUrls =
                                    value['payload'].mediaDetails,
                                rewardsData.title =
                                    value['payload'].contentMeta.title,
                                rewardsData.subtitle =
                                    value['payload'].contentMeta.meta,
                                rewardsData.recipientType =
                                    value['payload'].postRecipientType,
                                rewardsData.recipientDetails =
                                    value['payload'].postRecipientDetails,
                                rewardsData.id = value['payload'].id,
                                Navigator.of(ctx).pop({'payload': rewardsData})
                              }
                          });
                },
              ),
            ),
          ],
        );
      },
    );
  }

  Widget listItemBuilder(value, int index) {
    LoyaltyTypeItem item = value;
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () {},
      child: Padding(
          padding: const EdgeInsets.all(0),
          child: CommonListTile(
            title: item.loyaltyTypeName ?? "-",
            subTitle: item.loyaltyTypeDescription ?? "-",
            trailWidget: Checkbox(
              onChanged: (bool? value) {
                BlocProvider.of<CubitMain>(ctx)
                    .updateSelectedItem(value ?? false, index);
              },
              value: item.isSelected,
            ),
          )),
    );
  }
}
