import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:flutter/material.dart';

import 'package:loyalty_management_tool/bloc/rewards_module/pages/redemption_settings.dart';
import 'package:loyalty_management_tool/bloc/rewards_module/pages/rewards_list_page.dart';
import 'package:loyalty_management_tool/bloc/rewards_module/pages/rewards_recharge_history.dart';
import 'package:loyalty_management_tool/utils/basicInfo.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../../coupons_module/couponsPage.dart';
import '../../../models/enums/payments_enums.dart';
import '../../cubits/cubit_main.dart';
import 'loyalty_edit.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/custom_app_bar.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/locator.dart';

initializeScreeenUtils(BuildContext context) {}

class ManageRewardsPage extends StatefulWidget {
  String baseUrl;
  int userId;
  int businessId;
  String token;
  String currencySymbol;
  String appLanguageCode;
  String googleTranslationKey;
  String userName;
  String userImage;
  String baseUrlWithoutHttp;
  String timeFormat;
  String dateFormat;
  String mainColor;
  String googleMapKey;

  ManageRewardsPage(
      {Key? key,
      required this.baseUrl,
      required this.googleMapKey,
      required this.mainColor,
      required this.currencySymbol,
      required this.userId,
      required this.appLanguageCode,
      required this.googleTranslationKey,
      required this.token,
      required this.businessId,
      required this.userName,
      required this.userImage,
      required this.baseUrlWithoutHttp,
      required this.timeFormat,
      required this.dateFormat})
      : super(key: key);

  @override
  _ManageRewardsPage createState() => _ManageRewardsPage();
}

class _ManageRewardsPage extends State<ManageRewardsPage> {
  BasicInfo? basicInfo = BasicInfo();

  @override
  void initState() {
    super.initState();
    setBasicData();
  }

  setBasicData() async {
    basicInfo?.appLanguageCode = widget.appLanguageCode;
    basicInfo?.googleTransltionKey = widget.googleTranslationKey;
    basicInfo?.token = widget.token;
    basicInfo?.currencySymbol = widget.currencySymbol;
    basicInfo?.baseUrl = widget.baseUrl;
    basicInfo?.userId = widget.userId;
    basicInfo?.businessId = widget.businessId;
    basicInfo?.userName = widget.userName;
    basicInfo?.userImage = widget.userImage;
    basicInfo?.baseUrlWithoutHttp = widget.baseUrlWithoutHttp;
    basicInfo?.timeFormat = widget.timeFormat;
    basicInfo?.dateFormat = widget.dateFormat;
    basicInfo?.googleMapKey = widget.googleMapKey;
    AppColors.appMainColor = widget.mainColor;

    setState(() {});
  }

  late TextStyleElements styleElements;
  bool isSwitched = false;
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return SafeArea(
      child: Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: appAppBar().getCustomAppBar(context,
            appBarTitle: AppLocalizationsRewards.of(context) != null
                ? AppLocalizationsRewards.of(context)!
                    .translate("manage_loyalty")
                : "", onBackButtonPress: () {
          Navigator.pop(context);
        }),
        body: ListView(
          children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Card(
                color: HexColor(AppColors.greyBg),
                child: Padding(
                    padding: const EdgeInsets.only(
                        top: 16, left: 16, right: 16, bottom: 8),
                    child: RichText(
                      text: TextSpan(
                          style: styleElements
                              .captionThemeScalable(context)
                              .copyWith(fontWeight: FontWeight.w500),
                          children: [
                            TextSpan(
                                text:
                                    AppLocalizationsRewards.of(context) != null
                                        ? AppLocalizationsRewards.of(context)!
                                            .translate("loyalty_des")
                                        : ""),
                          ]),
                    )
                    // child: Text('Rooms are groups. You can create rooms to engage with more than one person together. Please click + on the top of the page to create new rooms.',style:
                    //   styleElements.captionThemeScalable(context).copyWith(fontWeight: FontWeight.w600),),
                    ),
              ),
            ),
            /*   InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RewardsListPage(
                          type: 'post',
                          messageTo: 'team',
                        ),
                      ),
                    ));
              },
              child: Card(
                elevation: 0,
                margin: EdgeInsets.only(left: 4, right: 4, bottom: 1),
                child: Padding(
                  padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                  child: ListTile(
                    leading: Image.asset(
                      'packages/loyalty_management_tool/assets/appimages/change-password.png',
                      width: 36,
                      height: 36,
                    ),
                    title: Text(
                      AppLocalizationsRewards.of(context)!=null?   AppLocalizationsRewards.of(context)!.translate("activate_lo"):"",
                      style: styleElements.subtitle1ThemeScalable(context),
                    ),
                    subtitle: Text(
                      AppLocalizationsRewards.of(context)!=null? AppLocalizationsRewards.of(context)!
                          .translate("activate_lo_des"):"",
                      style: styleElements.captionThemeScalable(context),
                    ),
                    trailing: Switch(
                      value: isSwitched,
                      onChanged: (value) {
                        setState(() {
                          isSwitched = value;
                        });
                      },
                    ),
                  ),
                ),
              ),
            ),
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RewardsListPage(
                          type: 'post',
                          messageTo: 'team',
                        ),
                      ),
                    ));
              },
              child: Card(
                elevation: 0,
                margin: EdgeInsets.only(left: 4, right: 4, bottom: 1),
                child: Padding(
                  padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                  child: ListTile(
                    leading: Image.asset(
                      'packages/loyalty_management_tool/assets/appimages/change-password.png',
                      width: 36,
                      height: 36,
                    ),
                    title: Text(
                      AppLocalizationsRewards.of(context)!=null?  AppLocalizationsRewards.of(context)!.translate("activate_cash"):"",
                      style: styleElements.subtitle1ThemeScalable(context),
                    ),
                    subtitle: Text(
                      AppLocalizationsRewards.of(context)!=null? AppLocalizationsRewards.of(context)!
                          .translate("activate_cash_des"):"",
                      style: styleElements.captionThemeScalable(context),
                    ),
                    trailing: Switch(
                      value: isSwitched,
                      onChanged: (value) {
                        setState(() {
                          isSwitched = value;
                        });
                      },
                    ),
                  ),
                ),
              ),
            ),
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RewardsListPage(
                          type: 'post',
                          messageTo: 'team',
                        ),
                      ),
                    ));
              },
              child: Card(
                elevation: 0,
                margin: EdgeInsets.only(left: 4, right: 4, bottom: 1),
                child: Padding(
                  padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                  child: ListTile(
                    leading: Image.asset(
                      'packages/loyalty_management_tool/assets/appimages/change-password.png',
                      width: 36,
                      height: 36,
                    ),
                    title: Text(
                      AppLocalizationsRewards.of(context)!=null?  AppLocalizationsRewards.of(context)!.translate("activate_api"):"",
                      style: styleElements.subtitle1ThemeScalable(context),
                    ),
                    subtitle: Text(
                      AppLocalizationsRewards.of(context)!=null?  AppLocalizationsRewards.of(context)!
                          .translate("activate_api_des"):"",
                      style: styleElements.captionThemeScalable(context),
                    ),
                    trailing: Switch(
                      value: isSwitched,
                      onChanged: (value) {
                        setState(() {
                          isSwitched = value;
                        });
                      },
                    ),
                  ),
                ),
              ),
            ),*/
            /*  InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: ManageCashLoyalty(),
                      ),
                    ));
              },
              child: Card(
                elevation: 0,
                margin: EdgeInsets.only(left: 4, right: 4, bottom: 1),
                child: Padding(
                  padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                  child: ListTile(
                    leading: Image.asset(
                      'packages/loyalty_management_tool/assets/appimages/change-password.png',
                      width: 36,
                      height: 36,
                    ),
                    title: Text(
                      AppLocalizationsRewards.of(context)!=null? AppLocalizationsRewards.of(context)!.translate("manage_cash"):"",
                      style: styleElements.subtitle1ThemeScalable(context),
                    ),
                    subtitle: Text(
                      AppLocalizationsRewards.of(context)!=null?   AppLocalizationsRewards.of(context)!
                          .translate("manage_cash_des"):"",
                      style: styleElements.captionThemeScalable(context),
                    ),
                    trailing: const Padding(
                      padding: EdgeInsets.all(8.0),
                      child: Icon(Icons.keyboard_arrow_right),
                    ),
                  ),
                ),
              ),
            ),*/
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                        builder: (BuildContext context) => LoyaltyEditPage()));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/loyalty-category.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_lo_cat")
                            : "",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_lo_cat_des")
                            : "",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: const Padding(
                        padding: EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),
            /*   InkWell(
              onTap: () {

              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/account-settings.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context)!=null?  AppLocalizationsRewards.of(context)!.translate("manage_lo"):"",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context)!=null?  AppLocalizationsRewards.of(context)!
                            .translate("manage_lo_des"):"",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: const Padding(
                        padding: EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),*/
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RedemptionSettingPage(),
                      ),
                    ));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/reddem.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_coin")
                            : "",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_coin_des")
                            : "",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: const Padding(
                        padding: EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RewardsListPage(
                          type: 'chat',
                          messageTo: 'partner',
                        ),
                      ),
                    ));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/manage-loyalty-program.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_lo_prog")
                            : "",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("manage_lo_prog_des")
                            : "",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child:
                            RewardsHistoryPage(type: PaymentEnums.coins.type),
                      ),
                    ));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/Buy-coins.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("buy_coin")
                            : "",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("buy_coin_des")
                            : "",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),
            InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => BlocProvider(
                        create: (_) => CubitMain(),
                        child: RewardsHistoryPage(type: PaymentEnums.cash.type),
                      ),
                    ));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/Recharge-wallet.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("recharge_wallet")
                            : "",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        AppLocalizationsRewards.of(context) != null
                            ? AppLocalizationsRewards.of(context)!
                                .translate("recharge_wallet_des")
                            : "",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: const Padding(
                        padding: EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),
        /*    InkWell(
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                        builder: (context) => CouponsPage(
                              isGift: false,
                            )));
              },
              child: Card(
                  elevation: 0,
                  margin: EdgeInsets.only(left: 4, right: 4, top: 1, bottom: 1),
                  child: Padding(
                    padding: EdgeInsets.only(top: 8, bottom: 8, left: 8),
                    child: ListTile(
                      leading: Image.asset(
                        'packages/loyalty_management_tool/assets/appimages/account-settings.png',
                        width: 36,
                        height: 36,
                      ),
                      title: Text(
                        "Rewards",
                        style: styleElements.subtitle1ThemeScalable(context),
                      ),
                      subtitle: Text(
                        "Rewards & Gifts",
                        style: styleElements.captionThemeScalable(context),
                      ),
                      trailing: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: Icon(Icons.keyboard_arrow_right),
                      ),
                    ),
                  )),
            ),*/
          ],
        ),
      ),
    );
  }
}
