import 'dart:convert';

import 'package:cashfree_pg/cashfree_pg.dart';
import 'package:event_bus/event_bus.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/models/enums/payments_enums.dart';
import 'package:loyalty_management_tool/utils/app_buttons.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:loyalty_management_tool/utils/toast_builder.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../../enums/payment_provider_type_enum.dart';
import '../../../events/history_refresh_event.dart';
import '../../../models/cash_coin_resppnse.dart';
import '../../../utils/basicInfo.dart';
import '../../../utils/colors.dart';
import '../../../utils/common_button.dart';
import '../../../utils/custom_app_bar.dart';
import '../../../utils/customcard.dart';
import '../../../utils/hexColors.dart';
import '../../../utils/localization.dart';
import '../../../utils/text_styling.dart';
import '../../common_widgets/account_details_billing_widget.dart';
import '../../common_widgets/header_widget.dart';
import '../../common_widgets/invoice_card.dart';
import '../../cubits/cubit_main.dart';
import '../../states/main_state.dart';

class PaymentPage extends StatelessWidget {
  final String title;
  final String type;
  final SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  BasicInfo? basicInfo = BasicInfo();
  final Function refreshCallBack;
  final Rows data;
  late MainState currentState;
  EventBus eventBus = EventBus();

  PaymentPage(
      {required this.title,
      required this.type,
      required this.data,
      required this.refreshCallBack});

  late TextStyleElements styleElements;

  GlobalKey<AccountDetailsBillingWidgetState> keyAddress = GlobalKey();

  void createCart(BuildContext context) {
    BlocProvider.of<CubitMain>(context).createCart(
      getOrderBody(null, "cart"),
      basicInfo!.createCart,
      context,
    );
  }

  void createOrder(BuildContext context) {
    BlocProvider.of<CubitMain>(context).createOrder(
        getOrderBody(null, "ordered"), context, basicInfo!.createOrder);
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    createOrder(context);
    return BlocBuilder<CubitMain, MainState>(
      builder: (context, state) {
        styleElements = TextStyleElements(context);
        if (state is CreateCartCallCompleted) {
          currentState = state;
          if (state.id != null) {
            BlocProvider.of<CubitMain>(context).createOrder(
                getOrderBody(state.id!, "ordered"),
                context,
                basicInfo!.createOrder);
          }
        } else if (state is PaymentStatusUpdated) {
          currentState = state;
          Navigator.pop(context);
          refreshCallBack();
        } else if (state is CallComplete) {
          currentState = state;
          ToastBuilder().showToast(
              AppLocalizationsRewards.of(context)!
                  .translate("oarder_successfull"),
              context,
              HexColor(AppColors.information));
        } else if (state is CreateOrderCompleted) {
          currentState = state;
        } else if (state is PaymentTokenReceived) {
          currentState = state;
          if (state.token != null && state.orderNumber != null) {
            makePayment(context, state.token ?? "", state.orderNumber);
          }
        } else if (state is CallFailed) {
          ToastBuilder().showToast(
              AppLocalizationsRewards.of(context)!.translate("something_wrong"),
              context,
              HexColor(AppColors.information));
        }

        return Scaffold(
          appBar: appAppBar().getCustomAppBar(
            context,
            appBarTitle: title,
            onBackButtonPress: () {
              Navigator.pop(context);
            },
          ),
          body: Stack(
            children: [
              ListView(
                children: [
                  appCard(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        HeaderWidget(
                            avatar: basicInfo!.userImage,
                            title: basicInfo!.userName,
                            subTitle: "Business Location ",
                            baseUrl: '',
                            refreshCallBack: () {
                              keyAddress.currentState!.dataFetch();
                            }),
                        BlocProvider(
                          create: (_) => CubitMain(),
                          child: AccountDetailsBillingWidget(
                            key: keyAddress,
                            title: basicInfo!.userName,
                          ),
                        ),
                      ],
                    ),
                  ),
                  appCard(
                      child: InvoiceCard(
                        type:type,
                    data: data,
                    title: "Title",
                    subTitle: type == PaymentEnums.cash.type ? "Cash" : "Coins",
                  )),
                ],
              ),
              Align(
                  alignment: Alignment.bottomCenter,
                  child: Container(
                      color: HexColor(AppColors.appMainColor),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Padding(
                            padding: const EdgeInsets.only(left: 8.0),
                            child: AppButtonCommon(
                              shape: RoundedRectangleBorder(
                                  borderRadius: BorderRadius.circular(18.0),
                                  side: BorderSide(
                                      color: HexColor(AppColors.appMainColor))),
                              onPressed: () async {},
                              color: HexColor(AppColors.appColorWhite),
                              child: Column(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  Text(
                                    AppLocalizationsRewards.of(context)!
                                        .translate('total_pay'),
                                    style: styleElements
                                        .captionThemeScalable(context)
                                        .copyWith(
                                            fontWeight: FontWeight.w600,
                                            color: HexColor(
                                                AppColors.appColorWhite)),
                                  ),
                                  Text(
                                    "₹" +
                                        data.payableAmount!
                                            .toStringAsFixed(2)
                                            .toString(),
                                    style: styleElements
                                        .headline6ThemeScalableW(context)
                                        .copyWith(
                                            fontWeight: FontWeight.w600,
                                            color: HexColor(
                                                AppColors.appColorWhite)),
                                  ),
                                ],
                              ),
                            ),
                          ),
                          Padding(
                            padding: const EdgeInsets.only(right: 8.0),
                            child: AppButtonCommon(
                              shape: RoundedRectangleBorder(
                                  borderRadius: BorderRadius.circular(18.0),
                                  side: BorderSide(
                                      color: HexColor(AppColors.appMainColor))),
                              onPressed: () async {
                                if (currentState is CreateOrderCompleted) {
                                  if ((currentState as CreateOrderCompleted)
                                          .orderId !=
                                      null) {
                                    BlocProvider.of<CubitMain>(context)
                                        .paymentToken(
                                            getPaymentTokenBody((currentState
                                                    as CreateOrderCompleted)
                                                .orderId),
                                            context,
                                            basicInfo!.paymentToken);
                                  }
                                }
                              },
                              color: HexColor(AppColors.appColorWhite),
                              child: state is CreatingCart
                                  ? SizedBox(
                                      height: 20,
                                      width: 20,
                                      child: CircularProgressIndicator(
                                        color:
                                            HexColor(AppColors.appColorWhite),
                                      ))
                                  : Row(
                                      mainAxisSize: MainAxisSize.min,
                                      mainAxisAlignment:
                                          MainAxisAlignment.center,
                                      children: [
                                        Text(
                                          AppLocalizationsRewards.of(context)!
                                              .translate('pay'),
                                          style: styleElements
                                              .headline6ThemeScalableW(context)
                                              .copyWith(
                                                  color: HexColor(
                                                      AppColors.appColorWhite)),
                                        ),
                                        Padding(
                                          padding: const EdgeInsets.all(2.0),
                                          child: Icon(
                                            Icons.arrow_forward_ios,
                                            color: HexColor(
                                                AppColors.appColorWhite),
                                          ),
                                        )
                                      ],
                                    ),
                            ),
                          ),
                        ],
                      )))
            ],
          ),
        );
      },
    );
  }

  String getCartViewBody() {
    return jsonEncode({
      "order_by_type": "business",
      "order_by_id": basicInfo!.businessId,
      "business_id": basicInfo!.businessId,
      "order_status": "cart"
    });
  }

  String updatePaymentStatusBody(String status, paymentId) {
    return jsonEncode({
      "id": (currentState as PaymentTokenReceived).orderId,
      "order_by_type": "business",
      "order_by_id": basicInfo!.businessId,
      "business_id": basicInfo!.businessId,
      "order_status": status,
      "payment_mode": "credit card",
      "payment_type": "online",
      "payment_reference_id": paymentId
    });
  }

  String getOrderBody(num? id, String orderType) {
    List<OrderItems>? orderItems = [];
    var obj = OrderItems();
    obj.itemType = type == PaymentEnums.cash.type ? "CASH" : "COIN";
    obj.itemDescription = type == PaymentEnums.cash.type ? "Cash" : "Coins";
    obj.totalAmount = data.payableAmount;
    obj.itemAmount = data.payableAmount;
    obj.taxAmount = data.taxAmount;

    obj.discountAmount = data.discountAmount;
    obj.isActive = true;
    obj.itemQuantity = data.cashOrCoins;
    data.id = id;

    data.businessId = basicInfo!.businessId;
    data.orderById = basicInfo!.businessId;
    data.orderByType = "business";
    data.orderStatus = orderType;
    orderItems.add(obj);
    data.orderItems = orderItems;
    data.orderAmount =
        type == PaymentEnums.cash.type ? data.payableAmount : 0.0;
    data.taxAmount = data.taxAmount;
    return jsonEncode(data);
  }

  String getPaymentTokenBody(orderId) {
    return jsonEncode({
      "id": orderId,
      "business_id": basicInfo!.businessId,
      "payment_gateway": PaymentTypeEnum.CASHFREE.type
    });
  }

  makePayment(BuildContext ctx, String token, String? orderId) {
    String customerName = "Savil Mehra";
    String orderNote = "Buy coins";
    String orderCurrency = "INR";
    String appId = "1562294b2ad9d27f0a55e894b8922651";
    String customerPhone = "9625678097";
    String customerEmail = "sample@gmail.com";
    String notifyUrl = "https://test.gocashfree.com/notify";

    Map<String, dynamic> inputParams = {
      "orderId": orderId,
      "orderAmount": data.payableAmount.toString(),
      "customerName": customerName,
      "orderNote": orderNote,
      "orderCurrency": orderCurrency,
      "appId": appId,
      "customerPhone": customerPhone,
      "customerEmail": customerEmail,
      "stage": "TEST",
      "tokenData": token,
      "notifyUrl": notifyUrl
    };

    Map<dynamic, dynamic> mapPaymentResult = {};
    CashfreePGSDK.doPayment(inputParams)
        .then((value) => value?.forEach((keys, value) {
              mapPaymentResult.putIfAbsent(keys, () => value);

              if (mapPaymentResult.containsKey("txStatus") &&
                  mapPaymentResult.containsKey("referenceId") &&
                  mapPaymentResult["txStatus"] == "SUCCESS") {
                BlocProvider.of<CubitMain>(ctx).paymentStatus(
                    updatePaymentStatusBody(
                        "paid", mapPaymentResult["referenceId"]),
                    ctx,
                    basicInfo!.updatePaymentStatus);
              } else if (mapPaymentResult.containsKey("txStatus") &&
                  mapPaymentResult.containsKey("referenceId") &&
                  mapPaymentResult.containsKey("orderId") &&
                  mapPaymentResult["txStatus"] == "FAILED") {
                showFailedMessage(ctx);
              }
            }));
  }

  showFailedMessage(BuildContext context) {
    AlertDialog alert = AlertDialog(
      title: Text("Payment Failed"),
      content: Text("Your transection is not completed , Please try again"),
      actions: [
        appTextButton(
            onPressed: () {
              if ((currentState as PaymentTokenReceived).token != null &&
                  (currentState as PaymentTokenReceived).orderNumber != null) {
                makePayment(
                    context,
                    (currentState as PaymentTokenReceived).token ?? "",
                    (currentState as PaymentTokenReceived).orderNumber);
              }
            },
            child: Text(
              "Retry",
            )),
      ],
    );

    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }
}
