import 'dart:convert';

import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:flutter/material.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/loyalty_benifits.dart';
import 'package:loyalty_management_tool/bloc/states/main_state.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/models/RedemptionResponse.dart';
import 'package:loyalty_management_tool/utils/app_user_list_tile.dart';
import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/common_helpdesk_sheet.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/empty_widget.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../../cubits/cubit_main.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/custom_app_bar.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';

import 'package:loyalty_management_tool/utils/text_styling.dart';

class RedemptionSettingPage extends StatefulWidget {
  @override
  _RedemptionSettingPage createState() => _RedemptionSettingPage();
}

class _RedemptionSettingPage extends State<RedemptionSettingPage> {
  BasicInfo? basicInfo = BasicInfo();
  LoadedWithData? loadedState;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      amountGift = TextField(
        controller: amountC,
        keyboardType: TextInputType.name,
        textCapitalization: TextCapitalization.words,
        onChanged: (text) {},
        decoration: InputDecoration(
            hintText:
                AppLocalizationsRewards.of(context)!.translate("enter_amt"),
            contentPadding: const EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText:
                AppLocalizationsRewards.of(context)!.translate("enter_amt")),
      );
      coinsPGift = TextField(
        controller: coinC,
        keyboardType: TextInputType.name,
        textCapitalization: TextCapitalization.words,
        onChanged: (text) {},
        decoration: InputDecoration(
            hintText: AppLocalizationsRewards.of(context)!.translate("coins"),
            contentPadding: EdgeInsets.only(left: 12, top: 16, bottom: 8),
            border:
                UnderlineInputBorder(borderRadius: BorderRadius.circular(12)),
            floatingLabelBehavior: FloatingLabelBehavior.auto,
            labelText:
                AppLocalizationsRewards.of(context)!.translate("en_per")),
      );
      BlocProvider.of<CubitMain>(context)
          .getData(getBody(), context, basicInfo!.dafoultRedemption, getData);
    });
  }

  dynamic getData(dynamic data) {
    return RedemptionResponse.fromJson(data);
  }

  String getBody() {
    return jsonEncode({
      "business_id": basicInfo!.businessId,
    });
  }

  late TextStyleElements styleElements;
  late TextEditingController cnt = TextEditingController();
  late TextEditingController newCont = TextEditingController();

  late TextEditingController amountC = TextEditingController();
  late TextEditingController coinC = TextEditingController();
  bool isSwitched = false;
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();

  Widget? amountGift;

  Widget? coinsPGift;

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return SafeArea(
      child: Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: appAppBar().getCustomAppBar(context,
            appBarTitle: AppLocalizationsRewards.of(context)!
                .translate("redemption_setting"), onBackButtonPress: () {
          Navigator.pop(context);
        }),
        body: appCard(child: _getBody()),
      ),
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);

      if (state is LoadingState) {
        return LoadingIndicator();
      } else if (state is LoadedWithData) {
        loadedState = state;
        return Column(
          children: [builder(state.data, true), builder(state.data, false)],
        );
      } else {
        return Container(
          child: Center(
            child: EmptyWidget(
                message:
                    AppLocalizationsRewards.of(context)!.translate("no_data")),
          ),
        );
      }
    });
  }

  Widget builder(RedemptionItem redemptionItem, bool isGift) {
    return InkWell(
      onTap: () {},
      child: Card(
        elevation: 0,
        margin: const EdgeInsets.only(left: 4, right: 4, bottom: 1),
        child: Padding(
          padding: const EdgeInsets.only(top: 8, bottom: 8, left: 8),
          child: appUserListTile(
            iconWidget: Image.asset(
              !isGift
                  ? 'packages/loyalty_management_tool/assets/appimages/reward.png'
                  : 'packages/loyalty_management_tool/assets/appimages/giftbox.png',
              width: 36,
              height: 36,
            ),
            title: AppLocalizationsRewards.of(context)!
                .translate(isGift ? "gift_red" : "disc_g"),
            subtitle1: isGift
                ? redemptionItem.giftCash.toString()
                : redemptionItem.couponDiscountPercentage.toString(),
            subtitle3: isGift
                ? redemptionItem.giftCoinsForCash.toString()
                : redemptionItem.couponCoinsForDiscountPercentage.toString(),
            trailingWidget: InkWell(
              onTap: () {
                openManageRedemption(isGift, redemptionItem);
              },
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Icon(
                  Icons.more_vert,
                  color: HexColor(AppColors.appColorBlack65),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  openManageRedemption(bool isGift, RedemptionItem redemptionItem) {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return Wrap(
          children: [
            CommonHelpdeskSheet(
              child: LoyaltyBenifits(
                isBothValueRequired: true,
                firstItemHint: AppLocalizationsRewards.of(context)!
                    .translate(isGift ? 'enter_amt' : "per_hint"),
                firstItemTag: AppLocalizationsRewards.of(context)!
                    .translate(isGift ? 'enter_amt' : "per_hint"),
                secondItemHint:
                    AppLocalizationsRewards.of(context)!.translate("en_per"),
                secondItemTag:
                    AppLocalizationsRewards.of(context)!.translate("en_per"),
                firstItemIcon: Icon(
                  Icons.build,
                  color: HexColor(AppColors.appColorRed),
                ),
                title: AppLocalizationsRewards.of(context)!
                    .translate(isGift ? "gift_red" : "mng_dsc"),
                subTitle: AppLocalizationsRewards.of(context)!
                    .translate(isGift ? "pg_jj" : "re_des"),
                secondItemIcon: Icon(
                  Icons.gamepad,
                  color: HexColor(AppColors.appColorRed),
                ),
                nextCallBack: (num cash, num coin) {
                  print("-----------ve");
                  updateSeetings(isGift, cash, coin, redemptionItem);
                },
              ),
            ),
          ],
        );
      },
    );
  }

  updateSeetings(bool isGift, num cash, num coin, RedemptionItem item) {
    RedemptionItem redemptionItem = RedemptionItem();
    redemptionItem.couponCoinsForDiscountPercentage =
        isGift ? item.couponCoinsForDiscountPercentage : cash;
    redemptionItem.couponDiscountPercentage =
        isGift ? item.couponDiscountPercentage : coin;
    redemptionItem.giftCash = isGift ? cash : item.giftCash;
    redemptionItem.giftCoinsForCash = isGift ? coin : item.giftCoinsForCash;
    redemptionItem.businessId = basicInfo!.businessId;
    BlocProvider.of<CubitMain>(context).updateRedemption(
        jsonEncode(redemptionItem),
        getBody(),
        context,
        basicInfo!.dafoultRedemptionUpdate,
        basicInfo!.dafoultRedemption,
        getData);
  }
}
