import 'dart:convert';
import 'package:loyalty_management_tool/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/paginator_list_view.dart';
import 'package:loyalty_management_tool/bloc/common_widgets/rewards_widget.dart';
import 'package:loyalty_management_tool/bloc/cubits/cubit_main.dart';
import 'package:loyalty_management_tool/bloc/states/main_state.dart';
import 'package:loyalty_management_tool/bloc/states/states.dart';
import 'package:loyalty_management_tool/models/create_post_payload.dart';
import 'package:loyalty_management_tool/models/rewards_list_response.dart';
import 'package:loyalty_management_tool/utils/appAvatar.dart';

import 'package:loyalty_management_tool/utils/basicInfo.dart';
import 'package:loyalty_management_tool/utils/colors.dart';
import 'package:loyalty_management_tool/utils/custom_app_bar.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';
import 'package:loyalty_management_tool/utils/empty_widget.dart';
import 'package:loyalty_management_tool/utils/hexColors.dart';
import 'package:loyalty_management_tool/utils/localization.dart';
import 'package:loyalty_management_tool/utils/locator.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/utils/resolutionenums.dart';
import 'package:loyalty_management_tool/utils/serviceTypeEnums.dart';
import 'package:loyalty_management_tool/utils/text_styling.dart';

import 'package:shared_preferences/shared_preferences.dart';

import 'loyalty_type_page_new.dart';

class RewardsListPage extends StatefulWidget {
  final String type;
  final String messageTo;

  RewardsListPage({required this.type, required this.messageTo});

  @override
  State<StatefulWidget> createState() => RewardsListPageState();
}

class RewardsListPageState extends State<RewardsListPage> {
  final scrollController = ScrollController();
  late TextStyleElements styleElements;
  List<dynamic> list = [];
  bool isLastPage = false;
  bool isActive = true;
  String? searchValue;
  BasicInfo? basicInfo = BasicInfo();
  List<PopupMenuItem> menuList = [];
  SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  late BuildContext ctx;

  Widget simplePopup() {
    // var name = headerData.title;
    return PopupMenuButton(
      icon: Icon(
        Icons.segment,
        size: 32,
        color: HexColor(AppColors.appColorBlack85),
      ),
      padding: EdgeInsets.only(right: 8),
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
      color: HexColor(AppColors.appColorBackground),
      itemBuilder: (context) => menuList,
      onSelected: (dynamic value) {
        switch (value) {
          case 'all':
            {
              isActive = true;
              BlocProvider.of<CubitMain>(ctx).page = 1;
              BlocProvider.of<CubitMain>(context).loadPage(
                  getBody(), context, basicInfo!.loyaltyProgramList, getData);
              break;
            }

          case 'inActive':
            {
              isActive = false;
              BlocProvider.of<CubitMain>(ctx).page = 1;
              BlocProvider.of<CubitMain>(context).loadPage(
                  getBody(), context, basicInfo!.loyaltyProgramList, getData);
              break;
            }
        }
      },
    );
  }

  @override
  void initState() {
    setupScrollController();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      menuList.add(PopupMenuItem(
          enabled: true,
          value: "all",
          child: getUiElement(
              AppLocalizationsRewards.of(context)!.translate("all_programs"),
              AppLocalizationsRewards.of(context)!
                  .translate("all_programs_des"),
              'assets/appimages/all.png',
              true)));
      menuList.add(PopupMenuItem(
          enabled: true,
          value: "inActive",
          child: getUiElement(
              AppLocalizationsRewards.of(context)!.translate("inActiveProgram"),
              AppLocalizationsRewards.of(context)!
                  .translate("inActiveProgram_des"),
              'assets/appimages/fake.png',
              true)));

      BlocProvider.of<CubitMain>(context)
          .loadPage(getBody(), context, basicInfo!.loyaltyProgramList, getData);
    });

    super.initState();
  }

  Widget getUiElement(
      String title, String subtitle, String assetimage, bool isAssets) {
    return appListCard(
      padding: EdgeInsets.all(8),
      margin: EdgeInsets.only(top: 2, bottom: 2, left: 4, right: 4),
      child: Row(
        children: [
          isAssets
              ? SizedBox(
                  height: 25,
                  width: 25,
                  child: Image(
                    image: AssetImage(assetimage),
                  ))
              : appAvatar(
                  key: UniqueKey(),
                  size: 20,
                  name: title,
                  isFullUrl: true,
                  imageUrl: basicInfo!.baseUrl + assetimage,
                  service_type: SERVICE_TYPE.BUSINESS,
                  resolution_type: RESOLUTION_TYPE.R64,
                ),
          Flexible(
            child: Padding(
              padding: EdgeInsets.only(left: 8.0),
              child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                      title,
                      style: styleElements.subtitle1ThemeScalable(context),
                    ),
                    Text(
                      subtitle,
                      maxLines: 2,
                      overflow: TextOverflow.ellipsis,
                    ),
                  ]),
            ),
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    this.ctx = context;
    styleElements = TextStyleElements(context);

    return Scaffold(
      appBar: appAppBar().getCustomAppBar(context,
          appBarTitle:
              AppLocalizationsRewards.of(context)!.translate("manage_lo_prog"),
          actions: [
            simplePopup(),
            IconButton(
                onPressed: () {
                  openDesiredPage();
                },
                icon: Icon(
                  Icons.add_circle,
                  color: HexColor(AppColors.blueApp),
                ))
          ], onBackButtonPress: () {
        Navigator.pop(context);
      }),
      body: _getBody(),
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      styleElements = TextStyleElements(context);
      if (state is LoadingStatePagination && state.isFirstFetch) {
        return LoadingIndicator();
      }
      bool isLoading = false;
      if (state is LoadingStatePagination) {
        list = state.list;
        isLoading = true;
      } else if (state is Loaded) {
        list = state.list;
        isLastPage = state.isLastPage;
      } else if (state is Empty) {
        return Center(
          child: EmptyWidget(
              message:
                  AppLocalizationsRewards.of(context)!.translate("no_data")),
        );
      }
      return Padding(
        padding: const EdgeInsets.only(top: 8.0),
        child: PaginatorListView(
          scrollController: scrollController,
          list: list,
          isLoading: isLoading,
          itemBuilder: getPostWidget,
        ),
      );
    });
  }

  Widget getPostWidget(value, index) {
    RewardsItem item = value;

    return RewardsWidget(
      item: item,
      deleteCallBack: (id) {
        if (id != null) {
          deleteReward(id);
        }
      },
      editCallBack: (item) {
        editRewards(item!);
      },
    );
  }

  void openDesiredPage() {
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => BlocProvider(
              create: (_) => CubitMain(),
              child: LoyaltyTypePageNew(
                item: null,
                key: null,
              )),
        )).then((value) => {
          if (value != null && value['payload'] != null)
            createPost(value['payload'], false)
        });
  }

  dynamic getData(dynamic data) {
    return RewardsListResponse.fromJson(data);
  }

  String getBody() {
    return jsonEncode({
      "business_id": basicInfo!.businessId,
      "page_size": 20,
      "status": isActive ? null : "deleted",
      "page_number": BlocProvider.of<CubitMain>(ctx).page,
      "app_type": "FOOPWORKS"
    });
  }

  void setupScrollController() {
    scrollController.addListener(() {
      if (scrollController.position.atEdge) {
        if (scrollController.position.pixels != 0) {
          if (!isLastPage) {
            BlocProvider.of<CubitMain>(ctx).loadPage(
                getBody(), ctx, basicInfo!.loyaltyProgramList, getData);
          }
        }
      }
    });
  }

  void deleteReward(int id) {
    BlocProvider.of<CubitMain>(ctx).page = 1;
    var body = jsonEncode({
      "id": id,
      "business_id": basicInfo!.businessId,
      "status": isActive ? "deleted" : "active"
    });
    context.read<CubitMain>().deleteRewards(
        body, getBody(), context, basicInfo!.loyaltyProgramList, getData);
  }

  void editRewards(RewardsItem data) {
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => BlocProvider(
              create: (_) => CubitMain(),
              child: LoyaltyTypePageNew(
                item: data,
                key: null,
              )),
        )).then((value) => {
          if (value != null && value['payload'] != null)
            value['payload'].id = data.id,
          createPost(value['payload'], true)
        });
  }

  void createPost(CreateRewardPayload? payload, bool isEdit) {
    BlocProvider.of<CubitMain>(ctx).page = 1;
    var body = jsonEncode(payload);
    context.read<CubitMain>().createPost(body, getBody(), context,
        basicInfo!.loyaltyProgramList, getData, isEdit);
  }
}
