import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_management_tool/utils/customcard.dart';

import '../../../models/couponheader.dart';
import '../../../models/couponlisting.dart';
import '../../../models/enums/payments_enums.dart';
import '../../../utils/basicInfo.dart';
import '../../../utils/custom_app_bar.dart';
import '../../../utils/localization.dart';
import '../../../utils/text_styling.dart';
import '../../common_widgets/recharge_card_widget.dart';
import '../../common_widgets/user_name_with_image_card.dart';
import '../../cubits/cubit_main.dart';
import '../../states/main_state.dart';
import '../../states/states.dart';
import 'ledger_history_page.dart';

class RewardsHistoryPage extends StatefulWidget {
  final String type;

  const RewardsHistoryPage({Key? key, required this.type}) : super(key: key);

  @override
  RewardsHistoryPageState createState() => RewardsHistoryPageState();
}

class RewardsHistoryPageState extends State<RewardsHistoryPage> {
  BasicInfo basicInfo = BasicInfo();
  late TextStyleElements styleElements;
  bool refreshHistory = false;

  GlobalKey<LeadgerHistoryPageState> key = GlobalKey();

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((_) {});
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      return SafeArea(
        child: Scaffold(
          appBar: appAppBar().getCustomAppBar(context,
              appBarTitle: AppLocalizationsRewards.of(context)!.translate(
                  widget.type == PaymentEnums.cash.type
                      ? 'recharge_wallet'
                      : 'recharge_coin'), onBackButtonPress: () {
            Navigator.pop(context);
          }),
          body: NestedScrollView(
            headerSliverBuilder: (context, value) {
              return [
                SliverToBoxAdapter(
                    child: appCard(
                        child: UserNameWithImageCard(
                            type: widget.type,
                            imageUrl: basicInfo.baseUrl + basicInfo.userImage,
                            subtitle: basicInfo.businessId.toString(),
                            callbackPicker: () {},
                            isFollow: true,
                            title: basicInfo.userName,
                            textFive: state is LoadedWithData
                                ? state.data.currentBalance
                                : "--",
                            textSix: "--",
                            textEight: state is LoadedWithData
                                ? state.data.isPostPaid
                                    ? "Post Paid"
                                    : "Pre-Paid"
                                : "--",
                            textSeven: state is LoadedWithData
                                ? state.data.status
                                : "--"))),
                SliverToBoxAdapter(
                    child: Padding(
                  padding: EdgeInsets.all(16.0),
                  child: Text(
                    "Your Cost",
                    overflow: TextOverflow.ellipsis,
                    style: styleElements
                        .headline6ThemeScalableW(context)
                        .copyWith(fontWeight: FontWeight.bold),
                    textAlign: TextAlign.left,
                  ),
                )),
                SliverToBoxAdapter(
                    child: appCard(
                        child: BlocProvider(
                  create: (_) => CubitMain(),
                  child: RechargeWalletCard(
                      refreshCallBack: () {
                        key.currentState!.refresh(widget.type);
                      },
                      textFive: '67',
                      textThree: 'Status',
                      type: widget.type,
                      textSix: '78',
                      callback: () {},
                      textTwo: 'For ₹1',
                      textEight: 'Post Paid',
                      textOne: 'Minimum',
                      textFour: 'Type',
                      textSeven: 'Active'),
                ))),
              ];
            },
            body: LeadgerHistoryPage(
                key: key, type: widget.type, refreshHistory: refreshHistory),
          ),
        ),
      );
    });
  }
}
