import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:shared_preferences/shared_preferences.dart';

import '../NetworkFile/calls.dart';
import '../bloc/common_widgets/dialogue_for_successfull_addition.dart';
import '../bloc/common_widgets/phone_upi_details.dart';
import '../bloc/common_widgets/upi_details_dialog.dart';
import '../bloc/cubits/cubit_main.dart';
import '../models/couponheader.dart';
import '../models/couponlisting.dart';
import '../models/enums/payments_enums.dart';
import '../utils/app_buttons.dart';
import '../utils/basicInfo.dart';
import '../utils/colors.dart';
import '../utils/commonComponents.dart';
import '../utils/common_helpdesk_sheet.dart';
import '../utils/hexColors.dart';
import '../utils/localization.dart';
import '../utils/text_styling.dart';
import '../utils/utility_class.dart';

class CouponPageHeaderCard extends StatefulWidget {
  final Function(int) changeCallBack;

  CouponPageHeaderCard({Key? key, required this.changeCallBack})
      : super(key: key);

  @override
  CouponPageHeaderCardState createState() => CouponPageHeaderCardState();
}

class CouponPageHeaderCardState extends State<CouponPageHeaderCard> {
  late SharedPreferences prefs;
  String? currentbalance = "0";
  String? lifetimeearned = "0";
  String? lifetimeburn = "0";

  String? currentbalanceCash = "0";
  String? lifetimeearnedCash = "0";
  String? lifetimeburnCash = "0";
  bool isCashSelected = false;
  late TextStyleElements styleElements;
  BasicInfo? basicInfo = BasicInfo();

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((_) => fetchData());
  }

  fetchData() async {
    prefs = await SharedPreferences.getInstance();
    CouponListRequest payload = CouponListRequest();
    payload.personId = basicInfo!.userId.toString();
    payload.ledgerType = PaymentEnums.coins.type;
    payload.businessId = basicInfo!.businessId.toString();
    payload.objectType = ObjectType.business.type;
    var data = jsonEncode(payload);
    Calls().call(data, context, basicInfo!.couponDetails).then((value) {
      var res = CouponHeaderResponse.fromJson(value);
      if (res.statusCode == basicInfo!.statusCode) {
        setState(() {
          currentbalance = res.rows!.currentBalance;
          lifetimeburn = res.rows!.lifetimeBurn;
          lifetimeearned = res.rows!.lifetimeEarned;
        });
      }
      fetchDataCash();
    }).catchError((onError) {
      print(onError);
    });
  }

  fetchDataCash() async {
    prefs = await SharedPreferences.getInstance();
    CouponListRequest payload = CouponListRequest();
    payload.personId = basicInfo!.userId.toString();
    payload.ledgerType = PaymentEnums.cash.type;
    payload.businessId = basicInfo!.businessId.toString();
    payload.objectType = ObjectType.business.type;

    var data = jsonEncode(payload);
    Calls().call(data, context, basicInfo!.couponDetails).then((value) {
      var res = CouponHeaderResponse.fromJson(value);
      if (res.statusCode == basicInfo!.statusCode) {
        setState(() {
          currentbalanceCash = res.rows!.currentBalance;
          lifetimeburnCash = res.rows!.lifetimeBurn;
          lifetimeearnedCash = res.rows!.lifetimeEarned;
        });
      }
    }).catchError((onError) {
      print(onError);
    });
  }

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);
    return Container(
      child: Column(
        children: [
          SizedBox(
            height: 60,
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Expanded(
                    child: InkWell(
                  onTap: () {
                    setState(() {
                      widget.changeCallBack(1);
                      isCashSelected = false;
                    });
                  },
                  child: Container(
                    decoration: BoxDecoration(
                      color: !isCashSelected
                          ? HexColor(AppColors.appMainColor)
                          : HexColor(AppColors.appColorWhite),
                      borderRadius: const BorderRadius.only(
                        topRight: Radius.circular(0.0),
                        bottomRight: Radius.circular(0.0),
                        bottomLeft: Radius.circular(0.0),
                        topLeft: Radius.circular(12.0),
                      ),
                    ),
                    height: 60,
                    child: Center(
                      child: Text(
                        AppLocalizationsRewards.of(context)!
                            .translate('app_coins'),
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(
                              color: !isCashSelected
                                  ? HexColor(AppColors.appColorWhite)
                                  : HexColor(AppColors.appMainColor),
                            ),
                      ),
                    ),
                  ),
                )),
                Expanded(
                    child: InkWell(
                  onTap: () {
                    setState(() {
                      isCashSelected = true;
                      widget.changeCallBack(2);
                    });
                  },
                  child: Container(
                    decoration: BoxDecoration(
                      color: isCashSelected
                          ? HexColor(AppColors.appMainColor)
                          : HexColor(AppColors.appColorWhite),
                      borderRadius: const BorderRadius.only(
                        topRight: Radius.circular(12.0),
                        bottomRight: Radius.circular(0.0),
                        bottomLeft: Radius.circular(0.0),
                        topLeft: Radius.circular(0.0),
                      ),
                    ),
                    height: 60,
                    child: Center(
                      child: Text(
                        AppLocalizationsRewards.of(context)!
                            .translate('cash_rewards'),
                        textAlign: TextAlign.center,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(
                              color: isCashSelected
                                  ? HexColor(AppColors.appColorWhite)
                                  : HexColor(AppColors.appMainColor),
                            ),
                      ),
                    ),
                  ),
                )),
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(left: 8.0, right: 8.0, top: 16.0),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Column(
                  children: [
                    Text(
                      AppLocalizationsRewards.of(context)!
                          .translate('current_balance'),
                      style: styleElements.bodyText2ThemeScalable(context),
                    ),
                    RichText(
                      text: TextSpan(children: [
                        !isCashSelected
                            ? WidgetSpan(
                                child: Padding(
                                  padding: const EdgeInsets.symmetric(
                                      horizontal: 1.0),
                                  child: SizedBox(
                                    height: 20,
                                    width: 20,
                                    child: Padding(
                                      padding: const EdgeInsets.all(2.0),
                                      child: Container(
                                        child: Image(
                                          image: AssetImage(
                                              'packages/loyalty_management_tool/assets/appimages/coins.png'),
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                              )
                            : TextSpan(
                                text: "₹",
                                style: styleElements
                                    .subtitle1ThemeScalable(context)),
                        TextSpan(
                            text: Utility().getCompatNumber(double.parse(
                                isCashSelected
                                    ? currentbalanceCash ?? "0.0"
                                    : currentbalance ??= "0.0")),
                            style:
                                styleElements.subtitle1ThemeScalable(context)),
                      ]),
                    )
                  ],
                )
              ],
            ),
          ),
          Visibility(
            visible: isCashSelected,
            child: Padding(
              padding: const EdgeInsets.only(left: 8.0, right: 16.0),
              child: Align(
                alignment: Alignment.centerRight,
                child: appElevatedButton(
                    elevation: 0,
                    boarderColor: HexColor(AppColors.appMainColor),
                    shape: StadiumBorder(
                        side: BorderSide(
                            color: HexColor(AppColors.appMainColor))),
                    onPressed: () {
                      _showGenderSelectionSheet();
                    },
                    child: Text(
                      AppLocalizationsRewards.of(context)!
                          .translate("withdraw"),
                      style: styleElements
                          .captionThemeScalable(context)
                          .copyWith(color: HexColor(AppColors.appMainColor)),
                    ),
                    color: HexColor(AppColors.appColorWhite)),
              ),
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(16),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Column(
                  children: [
                    Text(
                      AppLocalizationsRewards.of(context)!
                          .translate('lifetime_earned'),
                      style: styleElements.bodyText2ThemeScalable(context),
                    ),
                    RichText(
                      text: TextSpan(children: [
                        !isCashSelected
                            ? WidgetSpan(
                                child: Padding(
                                  padding: const EdgeInsets.symmetric(
                                      horizontal: 1.0),
                                  child: SizedBox(
                                    height: 20,
                                    width: 20,
                                    child: Padding(
                                      padding: const EdgeInsets.all(2.0),
                                      child: Container(
                                        child: const Image(
                                          image: AssetImage(
                                              'packages/loyalty_management_tool/assets/appimages/coins.png'),
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                              )
                            : TextSpan(
                                text: "₹",
                                style: styleElements
                                    .subtitle1ThemeScalable(context)),
                        TextSpan(
                            text: Utility().getCompatNumber(double.parse(
                                isCashSelected
                                    ? lifetimeearnedCash ?? "0.0"
                                    : lifetimeearned ??= "0.0")),
                            style:
                                styleElements.subtitle1ThemeScalable(context)),
                      ]),
                    )
                  ],
                ),
                Column(
                  children: [
                    Text(
                      AppLocalizationsRewards.of(context)!
                          .translate('lifetime_burn'),
                      style: styleElements.bodyText2ThemeScalable(context),
                    ),
                    RichText(
                      text: TextSpan(children: [
                        !isCashSelected
                            ? WidgetSpan(
                                child: Padding(
                                  padding: const EdgeInsets.symmetric(
                                      horizontal: 1.0),
                                  child: SizedBox(
                                    height: 20,
                                    width: 20,
                                    child: Padding(
                                      padding: const EdgeInsets.all(2.0),
                                      child: Container(
                                        child: Image(
                                          image: AssetImage(
                                              'packages/loyalty_management_tool/assets/appimages/coins.png'),
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                              )
                            : TextSpan(
                                text: "₹",
                                style: styleElements
                                    .subtitle1ThemeScalable(context)),
                        TextSpan(
                            text: Utility().getCompatNumber(double.parse(
                                isCashSelected
                                    ? lifetimeburnCash ?? "0.0"
                                    : lifetimeburn ??= "0.0")),
                            style:
                                styleElements.subtitle1ThemeScalable(context)),
                      ]),
                    )
                  ],
                )
              ],
            ),
          ),
        ],
      ),
    );
  }

  void _showGenderSelectionSheet() {
    int selection = 0;
    showModalBottomSheet(
      context: context,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return CommonHelpdeskSheet(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              Padding(
                padding: const EdgeInsets.only(
                    bottom: 30.0, left: 16, right: 16, top: 8),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  //Center Row contents horizontally,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  //Center Row contents vertically,

                  children: [
                    Expanded(
                      child: Align(
                        alignment: Alignment.center,
                        child: Container(
                          child: Text(
                            AppLocalizationsRewards.of(context)!
                                .translate('paymet_option'),
                            textAlign: TextAlign.center,
                            style: styleElements
                                .subtitle1ThemeScalable(context)
                                .copyWith(fontWeight: FontWeight.bold),
                          ),
                        ),
                      ),
                    ),
                    Align(
                      alignment: Alignment.centerRight,
                      child: InkWell(
                        onTap: () {
                          _UpiIdSheet();
                        },
                        child: Container(
                          child: Text(
                            AppLocalizationsRewards.of(context)!
                                .translate("next"),
                            textAlign: TextAlign.center,
                            style: styleElements
                                .subtitle1ThemeScalable(context)
                                .copyWith(
                                    fontWeight: FontWeight.bold,
                                    color: HexColor(AppColors.blueApp)),
                          ),
                        ),
                      ),
                    )
                  ],
                ),
              ),
              Padding(
                padding: const EdgeInsets.all(12),
                child: Text(
                  AppLocalizationsRewards.of(context)!.translate("pl_payment"),
                  textAlign: TextAlign.center,
                  style: styleElements.captionThemeScalable(context),
                ),
              ),
              Padding(
                padding: const EdgeInsets.only(top: 32.0, bottom: 24),
                child: SelectableWidget(callback: (value) {
                  selection = value;
                }),
              ),
            ],
          ),
        );
      },
    );
  }

  void _PhoneUpiIdSheet() {
    int selection = 0;
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return CommonHelpdeskSheet(
          child: BlocProvider(
              create: (_) => CubitMain(),
              child: PhoneUpiDetailsDilaog(
                nextCallBack: (s) {
                  _success();
                },
              )),
        );
      },
    );
  }

  void _UpiIdSheet() {
    int selection = 0;
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(12), topRight: Radius.circular(12))),
      builder: (BuildContext context) {
        return CommonHelpdeskSheet(
          child: BlocProvider(
              create: (_) => CubitMain(),
              child: UpiDetailsDialog(
                nextCallBack: (s) {
                  _PhoneUpiIdSheet();
                },
              )),
        );
      },
    );
  }

  void _success() {
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (BuildContext context) => CustomDialogue(
              title: 'Payment Successfull',
              subtitle: 'Your payment is successfully paid to 986748438',
              type: 'Cash',
              isVerified: true,
            ));
  }
}

class SelectableWidget extends StatefulWidget {
  final Function(int) callback;

  SelectableWidget({required this.callback});

  @override
  SelectableWidgetState createState() => SelectableWidgetState();
}

class SelectableWidgetState extends State<SelectableWidget> {
  int selection = 0;

  @override
  Widget build(BuildContext context) {
    var styleElements = TextStyleElements(context);
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: [
        InkWell(
          onTap: () {
            widget.callback(1);
            setState(() {
              selection = 1;
            });
          },
          child: Container(
            width: 130,
            padding: const EdgeInsets.all(16),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(12),
                boxShadow: [CommonComponents().getShadowforBox_03_9()],
                color: selection == 1
                    ? HexColor(AppColors.appMainColor).withOpacity(0.2)
                    : HexColor(AppColors.appColorWhite)),
            child: Column(
              children: [
                Icon(
                  Icons.phone_bluetooth_speaker,
                  size: 45,
                ),
                Text(
                  "Phone",
                  style: styleElements.subtitle1ThemeScalable(context),
                )
              ],
            ),
          ),
        ),
        InkWell(
          onTap: () {
            widget.callback(2);
            setState(() {
              selection = 2;
            });
          },
          child: Container(
            width: 130,
            padding: const EdgeInsets.all(16),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(12),
                boxShadow: [CommonComponents().getShadowforBox_03_9()],
                color: selection == 2
                    ? HexColor(AppColors.appMainColor).withOpacity(0.2)
                    : HexColor(AppColors.appColorWhite)),
            child: Column(
              children: [
                SizedBox(
                  height: 45,
                  width: 45,
                  child: Padding(
                    padding: const EdgeInsets.all(2.0),
                    child: Container(
                      child: Image(
                        image: AssetImage(
                            'packages/loyalty_management_tool/assets/appimages/upi.png'),
                      ),
                    ),
                  ),
                ),
                Text(
                  "Upi",
                  style: styleElements.subtitle1ThemeScalable(context),
                )
              ],
            ),
          ),
        ),
      ],
    );
  }
}
