import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:loyalty_management_tool/utils/localization.dart';

import '../models/couponlisting.dart';
import '../utils/basicInfo.dart';
import '../utils/colors.dart';
import '../utils/customcard.dart';
import '../utils/hexColors.dart';
import '../utils/resolutionenums.dart';
import '../utils/serviceTypeEnums.dart';
import '../utils/text_styling.dart';
import '../utils/utility_class.dart';

class GiftItemCard extends StatelessWidget {
  final String? imageUrl;
  final String? title;
  final String? subtitle1;
  final String? subtitle2;
  BasicInfo? basicInfo = BasicInfo();

  GiftItemCard({this.imageUrl, this.title, this.subtitle1, this.subtitle2});

  @override
  Widget build(BuildContext context) {
    var styleElements = TextStyleElements(context);
    return Container(
      child: Center(
        child: Container(
          margin: EdgeInsets.all(4),
          child: appListCard(
            margin: EdgeInsets.all(0),
            padding: EdgeInsets.all(0),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                AspectRatio(
                  aspectRatio: 1,
                  child: Container(
                    decoration: BoxDecoration(
                        borderRadius: BorderRadius.only(
                            topLeft: Radius.circular(16),
                            topRight: Radius.circular(16)),
                        image: DecorationImage(
                            image: CachedNetworkImageProvider(
                              imageUrl != null
                                  ? basicInfo!.baseUrl + imageUrl!
                                  : Utility().getUrlForImage("",
                                      RESOLUTION_TYPE.R64, SERVICE_TYPE.POST),
                            ),
                            fit: BoxFit.cover)),
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        title ?? "",
                        style: styleElements.subtitle1ThemeScalable(context),
                        maxLines: 1,
                      ),
                      Row(
                        children: [
                          Spacer(),
                          Text(
                            subtitle1 ?? "",
                            style:
                                styleElements.subtitle2ThemeScalable(context),
                            maxLines: 1,
                          ),
                        ],
                      ),
                      Row(
                        children: [
                          Spacer(),
                          Text(
                            subtitle2 ?? "",
                            style: styleElements.captionThemeScalable(context),
                            maxLines: 1,
                          ),
                        ],
                      )
                    ],
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class GiftCardBanner extends StatelessWidget {
  final CouponListItem? data;
  BasicInfo? basicInfo = BasicInfo();

  GiftCardBanner({this.data});

  @override
  Widget build(BuildContext context) {
    var styleElements = TextStyleElements(context);
    return SizedBox(
      height: 80,
      child: Container(
        color: HexColor(AppColors.appColorBackground),
        child: Row(
          children: [
            Expanded(
              child: Container(
                padding: EdgeInsets.all(8),
                child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        data!.couponProvider!,
                        style: styleElements
                            .subtitle1ThemeScalable(context)
                            .copyWith(fontWeight: FontWeight.bold),
                      ),
                      Text(
                        AppLocalizationsRewards.of(context)!.translate(
                            'gift_voucher',
                            arguments: {"company_name": data!.couponProvider}),
                        style: styleElements.bodyText2ThemeScalable(context),
                        maxLines: 1,
                        overflow: TextOverflow.ellipsis,
                      ),
                      Text(AppLocalizationsRewards.of(context)!
                          .translate('congratulations')),
                    ]),
              ),
            ),
            AspectRatio(
              aspectRatio: 1,
              child: Container(
                decoration: BoxDecoration(
                    image: DecorationImage(
                        image: CachedNetworkImageProvider(
                          basicInfo!.baseUrl + data!.couponImage.toString(),
                        ),
                        fit: BoxFit.cover)),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
