import 'package:flutter/material.dart';

import 'colors.dart';
import 'hexColors.dart';

class CustomTabView extends StatefulWidget {
  final int itemCount;
  final IndexedWidgetBuilder tabBuilder;
  final IndexedWidgetBuilder pageBuilder;
  final Widget? stub;
  final ValueChanged<int?>? onPositionChange;
  final ValueChanged<double>? onScroll;
  final int? currentPosition;
  final Null Function()? callback;
  final EdgeInsetsGeometry? marginTop;
  final bool? isTabVisible;
  final bool? isSwipeDisabled;

  CustomTabView({
    required this.itemCount,
    required this.tabBuilder,
    required this.pageBuilder,
    this.stub,
    this.onPositionChange,
    this.onScroll,
    this.marginTop,
    this.callback,
    this.isSwipeDisabled = false,
    this.isTabVisible,
    this.currentPosition,
  });

  @override
  _CustomTabsState createState() => _CustomTabsState(callback);
}

class _CustomTabsState extends State<CustomTabView>
    with TickerProviderStateMixin {
  TabController? controller;
  int? _currentCount;
  int? _currentPosition;
  final Null Function()? callback;

  @override
  void initState() {
    _currentPosition = widget.currentPosition ?? 0;

    controller = TabController(
      length: widget.itemCount,
      vsync: this,
      initialIndex: _currentPosition!,
    );
    controller!.addListener(onPositionChange);
    controller!.animation!.addListener(onScroll);
    _currentCount = widget.itemCount;
    super.initState();
  }

  @override
  void didUpdateWidget(CustomTabView oldWidget) {
    if (_currentCount != widget.itemCount) {
      controller!.animation!.removeListener(onScroll);
      controller!.removeListener(onPositionChange);
      controller!.dispose();

      if (widget.currentPosition != null) {
        _currentPosition = widget.currentPosition;
      }

      if (_currentPosition! > widget.itemCount - 1) {
        _currentPosition = widget.itemCount - 1;
        _currentPosition = _currentPosition! < 0 ? 0 : _currentPosition;
        if (widget.onPositionChange is ValueChanged<int>) {
          WidgetsBinding.instance!.addPostFrameCallback((_) {
            if (mounted) {
              widget.onPositionChange!(_currentPosition);
            }
          });
        }
      }

      _currentCount = widget.itemCount;
      setState(() {
        controller = TabController(
          length: widget.itemCount,
          vsync: this,
          initialIndex: _currentPosition!,
        );
        controller!.addListener(onPositionChange);
        controller!.animation!.addListener(onScroll);
      });
    } else if (widget.currentPosition != null) {
      controller!.animateTo(widget.currentPosition!);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    controller!.animation!.removeListener(onScroll);
    controller!.removeListener(onPositionChange);
    controller!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.itemCount < 1) return widget.stub ?? Container();

    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Visibility(
          visible: widget.isTabVisible != null ? widget.isTabVisible! : true,
          child: Container(
            margin: widget.marginTop,
            alignment: Alignment.center,
            child: TabBar(
              isScrollable: true,
              controller: controller,
              labelPadding: EdgeInsets.symmetric(horizontal: 3.0),
              indicatorColor: HexColor(AppColors.appColorTransparent),
              labelColor: HexColor(AppColors.appMainColor),
              unselectedLabelColor: Theme.of(context).hintColor,
              indicator: BoxDecoration(
                border: Border(
                  bottom: BorderSide(
                    color: HexColor(AppColors.appColorTransparent),
                    width: 0,
                  ),
                ),
              ),
              tabs: List.generate(
                widget.itemCount,
                (index) => widget.tabBuilder(context, index),
              ),
            ),
          ),
        ),
        Expanded(
          child: TabBarView(
            controller: controller,
            physics: widget.isSwipeDisabled!
                ? NeverScrollableScrollPhysics()
                : BouncingScrollPhysics(),
            children: List.generate(
              widget.itemCount,
              (index) => widget.pageBuilder(context, index),
            ),
          ),
        ),
      ],
    );
  }

  onPositionChange() {
    if (!widget.isSwipeDisabled!) {
      if (!controller!.indexIsChanging) {
        _currentPosition = controller!.index;
        if (widget.onPositionChange is ValueChanged<int>) {
          widget.onPositionChange!(_currentPosition);
        }
      }
    }
  }

  onScroll() {
    if (!widget.isSwipeDisabled!) {
      if (widget.onScroll is ValueChanged<double>) {
        widget.onScroll!(controller!.animation!.value);
      }
    }
  }

  _CustomTabsState(this.callback);
} // ignore: must_be_immutable
