import 'dart:async';

import 'package:flutter/material.dart';

import '../bloc/common_widgets/customgridDelegate.dart';

typedef ItemBuilder = Widget Function(dynamic itemData, int index);

typedef Item = dynamic Function(dynamic itemData);

class PaginatorListView extends StatelessWidget {
  final bool isLoading;

  final bool isVertical;
  final bool isScrollAble;
  final bool isGridView;
  final double gridViewHeight;
  final int crossAxisCount;

  final bool isGridWithTwoRows;
  final ScrollController scrollController;
  final List<dynamic> list;

  final ItemBuilder itemBuilder;

  PaginatorListView(
      {Key? key,
      this.gridViewHeight = 250,
      this.crossAxisCount = 2,
      this.isGridWithTwoRows = false,
      required this.isScrollAble,
      required this.isGridView,
      required this.isVertical,
      required this.list,
      required this.itemBuilder,
      required this.scrollController,
      required this.isLoading})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return isGridWithTwoRows
        ? GridView.builder(
            scrollDirection: Axis.horizontal,
            gridDelegate: const SliverGridDelegateWithMaxCrossAxisExtent(
                maxCrossAxisExtent: 200,
                childAspectRatio: 0.5,
                crossAxisSpacing: 10,
                mainAxisSpacing: 10),
            itemBuilder: (context, index) {
              if (index < list.length) {
                return _itemBuilder(list[index], index);
              } else {
                Timer(const Duration(milliseconds: 30), () {
                  scrollController
                      .jumpTo(scrollController.position.maxScrollExtent);
                });

                return _loadingIndicator();
              }
            },
            itemCount: list.length + (isLoading ? 1 : 0),
          )
        : isGridView
            ? GridView.builder(
                gridDelegate:
                    SliverGridDelegateWithFixedCrossAxisCountAndFixedHeight(
                        crossAxisCount: crossAxisCount, height: gridViewHeight),
                itemBuilder: (context, index) {
                  if (index < list.length) {
                    return _itemBuilder(list[index], index);
                  } else {
                    Timer(const Duration(milliseconds: 30), () {
                      scrollController
                          .jumpTo(scrollController.position.maxScrollExtent);
                    });

                    return _loadingIndicator();
                  }
                },
                itemCount: list.length + (isLoading ? 1 : 0),
              )
            : ListView.builder(
                controller: scrollController,
                shrinkWrap: true,
                physics: const ScrollPhysics(),
                scrollDirection: isVertical ? Axis.vertical : Axis.horizontal,
                itemBuilder: (context, index) {
                  if (index < list.length) {
                    return _itemBuilder(list[index], index);
                  } else {
                    Timer(const Duration(milliseconds: 30), () {
                      scrollController
                          .jumpTo(scrollController.position.maxScrollExtent);
                    });

                    return _loadingIndicator();
                  }
                },
                itemCount: list.length + (isLoading ? 1 : 0),
              );
  }

  Widget _loadingIndicator() {
    return const Padding(
      padding: EdgeInsets.all(8.0),
      child: Center(child: CircularProgressIndicator()),
    );
  }

  Widget _itemBuilder(dynamic data, int index) {
    return itemBuilder(data, index);
  }
}
